@@ -22,25 +22,17 @@ from __future__ import with_statement  # Required in 2.5
 from __future__ import print_function
 import subprocess
 import os
-import fnmatch
-import gzip
-import re
-import commands
-import getopt
 import sys
-import signal
 import time
-import struct
 import random
 from random import choice
-from subprocess import Popen, PIPE, STDOUT
-# from optparse import OptionParser
 import optparse
 import glob
+import resource
 from verifier import *
 from functools import partial
 
-print("our CWD is: %s files here: %s" % (os.getcwd(), glob.glob("*")) )
+print("our CWD is: %s files here: %s" % (os.getcwd(), glob.glob("*")))
 sys.path.append(os.getcwd())
 print("our sys.path is", sys.path)
 
@@ -55,58 +47,75 @@ class PlainHelpFormatter(optparse.IndentedHelpFormatter):
         else:
             return ""
 
+
 usage = "usage: %prog [options] --fuzz/--regtest/--checkdir/filetocheck"
 desc = """Fuzz the solver with fuzz-generator: ./fuzz_test.py
 """
 
-parser = optparse.OptionParser(usage=usage, description=desc,
-                               formatter=PlainHelpFormatter())
-parser.add_option("--exec", metavar="SOLVER", dest="solver",
-                  default="../../build/cryptominisat5",
-                  help="SAT solver executable. Default: %default")
 
-parser.add_option("--extraopts", "-e", metavar="OPTS",
-                  dest="extra_options", default="",
-                  help="Extra options to give to SAT solver")
+def set_up_parser():
+    parser = optparse.OptionParser(usage=usage, description=desc,
+                                   formatter=PlainHelpFormatter())
+    parser.add_option("--exec", metavar="SOLVER", dest="solver",
+                      default="../../build/cryptominisat5",
+                      help="SAT solver executable. Default: %default")
 
-parser.add_option("--verbose", "-v", action="store_true", default=False,
-                  dest="verbose", help="Print more output")
+    parser.add_option("--extraopts", "-e", metavar="OPTS",
+                      dest="extra_options", default="",
+                      help="Extra options to give to SAT solver")
 
-# for fuzz-testing
-parser.add_option("--seed", dest="fuzz_seed_start",
-                  help="Fuzz test start seed. Otherwise, random seed is picked"
-                  " (printed to console)", type=int)
+    parser.add_option("--verbose", "-v", action="store_true", default=False,
+                      dest="verbose", help="Print more output")
 
-parser.add_option("--fuzzlim", dest="fuzz_test_lim", type=int,
-                  help="Number of fuzz tests to run"
-                  )
-parser.add_option("--novalgrind", dest="novalgrind", default=False,
-                  action="store_true", help="No valgrind installed")
+    # for fuzz-testing
+    parser.add_option("--seed", dest="fuzz_seed_start",
+                      help="Fuzz test start seed. Otherwise, random seed is picked"
+                      " (printed to console)", type=int)
 
-parser.add_option("--small", dest="small", default=False,
-                  action="store_true",
-                  help="Don't run 'large' fuzzer"
-                  " (may mem-out on smaller systems)")
+    parser.add_option("--fuzzlim", dest="fuzz_test_lim", type=int,
+                      help="Number of fuzz tests to run"
+                      )
+    parser.add_option("--novalgrind", dest="novalgrind", default=False,
+                      action="store_true", help="No valgrind installed")
+    parser.add_option("--valgrindfreq", dest="valgrind_freq", type=int,
+                      default=10, help="1 out of X times valgrind will be used. Default: %default in 1")
 
-parser.add_option("--sqlite", dest="sqlite", default=False,
-                  action="store_true", help="Test SQLite dumping")
+    parser.add_option("--small", dest="small", default=False,
+                      action="store_true",
+                      help="Don't run 'large' fuzzer"
+                      " (may mem-out on smaller systems)")
+    parser.add_option("--gauss", dest="gauss", default=False,
+                      action="store_true",
+                      help="Concentrate fuzzing gauss")
+    parser.add_option("--indep", dest="only_indep", default=False,
+                      action="store_true",
+                      help="Concentrate fuzzing independent variables")
+    parser.add_option("--dump", dest="only_dump", default=False,
+                      action="store_true",
+                      help="Concentrate fuzzing dumped clauses")
 
-parser.add_option("--gauss", dest="test_gauss", default=False,
-                  action="store_true", help="Test gauss too")
+    parser.add_option("--maxth", "-m", dest="max_threads", default=100,
+                      type=int, help="Max number of threads")
 
-parser.add_option("--tout", "-t", dest="maxtime", type=int, default=35,
-                  help="Max time to run. Default: %default")
+    parser.add_option("--tout", "-t", dest="maxtime", type=int, default=35,
+                      help="Max time to run. Default: %default")
 
-parser.add_option("--textra", dest="maxtimediff", type=int, default=10,
-                  help="Extra time on top of timeout for processing."
-                  " Default: %default")
-
-(options, args) = parser.parse_args()
+    parser.add_option("--textra", dest="maxtimediff", type=int, default=10,
+                      help="Extra time on top of timeout for processing."
+                      " Default: %default")
+    return parser
 
 
-def fuzzer_call_failed():
+def fuzzer_call_failed(fname):
     print("OOps, fuzzer executable call failed!")
     print("Did you build with cmake -DENABLE_TESTING=ON? Did you do git submodules init & update?")
+    print("Here is the output:")
+
+    print("**** ----- ****")
+    with open(fname, "r") as a:
+        for x in a:
+            print(x.strip())
+    print("**** ----- ****")
     exit(-1)
 
 
@@ -136,7 +145,7 @@ class create_fuzz:
             # sometimes just fuzz with all SAT problems
             fixed = random.getrandbits(1) == 1
 
-            for i in range(random.randrange(2, 4)):
+            for _ in range(random.randrange(2, 4)):
                 fname2 = unique_file("fuzzTest")
                 fnames_multi.append(fname2)
 
@@ -152,9 +161,9 @@ class create_fuzz:
                 print("fuzzer2 used: %s" % fuzzer2)
                 call = self.call_from_fuzzer(fuzzer2, fname2)
                 print("calling sub-fuzzer: %s" % call)
-                status, _ = commands.getstatusoutput(call)
+                status = subprocess.call(call, shell=True)
                 if status != 0:
-                    fuzzer_call_failed()
+                    fuzzer_call_failed(fname2)
 
             # construct multi-fuzzer call
             call = ""
@@ -181,46 +190,27 @@ def file_exists(fname):
 
 def print_version():
     command = options.solver + " --version"
-    p = subprocess.Popen(command.rsplit(), stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
+    if options.verbose:
+        print("Executing: %s" % command)
+    p = subprocess.Popen(command.rsplit(), stderr=subprocess.STDOUT,
+            stdout=subprocess.PIPE, universal_newlines=True)
+
     consoleOutput, err = p.communicate()
     print("Version values: %s" % consoleOutput.strip())
 
-fuzzers_noxor = [
-    ["../../build/tests/sha1-sat/sha1-gen --attack preimage --rounds 20",
-     "--hash-bits", "--seed"],
-    ["../../build/tests/sha1-sat/sha1-gen --attack preimage --zero --message-bits 400 --rounds 8 --hash-bits 60",
-     "--seed"],
-    # ["build/cnf-fuzz-nossum"],
-    ["../../build/tests/cnf-utils/largefuzzer"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/sgen4 -unsat -n 50", "-s"],
-    ["../../build/tests/cnf-utils//sgen4 -sat -n 50", "-s"],
-    ["../../utils/cnf-utils/cnf-fuzz-brummayer.py", "-s"],
-    ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
-    ["../../utils/cnf-utils/multipart.py", "special"]
-]
-fuzzers_xor = [
-    ["../../utils/cnf-utils/xortester.py", "--seed"],
-    ["../../build/tests/sha1-sat/sha1-gen --xor --attack preimage --rounds 21",
-     "--hash-bits", "--seed"],
-]
-
 
 class Tester:
 
     def __init__(self):
         self.ignoreNoSolution = False
-        self.extra_options_if_supported = self.list_options_if_supported(
-            ["xor", "autodisablegauss"])
+        self.extra_opts_supported = self.list_options_if_supported(
+            ["xor", "autodisablegauss", "sql", "clid"])
         self.sol_parser = solution_parser(options)
+        self.sqlitedbfname = None
+        self.clid_added = False
+        self.only_indep = False
+        self.indep_vars = []
+        self.dump_red = None
 
     def list_options_if_supported(self, tocheck):
         ret = []
@@ -234,7 +224,10 @@ class Tester:
         command = options.solver
         command += " --hhelp"
         p = subprocess.Popen(
-            command.rsplit(), stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
+            command.rsplit(), stderr=subprocess.STDOUT,
+            stdout=subprocess.PIPE,
+            universal_newlines=True)
+
         consoleOutput, err = p.communicate()
 
         for l in consoleOutput.split("\n"):
@@ -244,15 +237,85 @@ class Tester:
 
         return False
 
+    def create_rnd_sched(self, string_list):
+        opts = string_list.split(",")
+        opts = [a.strip(" ") for a in opts]
+        opts = sorted(list(set(opts)))
+        if options.verbose:
+            print("available schedule options: %s" % opts)
+
+        sched = []
+        for _ in range(int(random.gammavariate(12, 0.7))):
+            sched.append(random.choice(opts))
+
+        # just so that XOR is really found and used, so we can fuzz it
+        if "autodisablegauss" in self.extra_opts_supported:
+            if random.choice([False, True, True, True]) and self.this_gauss_on:
+                sched.append("occ-xor")
+
+        return sched
+
+    def rnd_schedule_all(self, preproc):
+        sched_opts = "handle-comps,"
+        sched_opts += "scc-vrepl, cache-clean, cache-tryboth,"
+        sched_opts += "sub-impl, intree-probe, probe,"
+        sched_opts += "sub-str-cls-with-bin, distill-cls, scc-vrepl, sub-impl,"
+        sched_opts += "sub-cls-with-bin,"
+        sched_opts += "str-impl, cache-clean, sub-str-cls-with-bin, distill-cls, scc-vrepl,"
+        sched_opts += "occ-backw-sub-str, occ-xor, occ-clean-implicit, occ-bve, occ-bva,"
+        sched_opts += "check-cache-size, renumber"
+
+        # type of schedule
+        cmd = ""
+        sched = ",".join(self.create_rnd_sched(sched_opts))
+        if sched != "" and not preproc:
+            cmd += "--schedule %s " % sched
+
+        sched = ",".join(self.create_rnd_sched(sched_opts))
+        if sched != "":
+            cmd += "--preschedule %s " % sched
+
+        return cmd
+
     def random_options(self, preproc=False):
+        self.sqlitedbfname = None
+        self.clid_added = False
         cmd = " --zero-exit-status "
 
-        if random.choice([True, False]):
-            cmd += " --reconf %d " % random.choice([3, 6, 7, 12, 13, 14])
-            cmd += " --undef %d " % random.choice([0, 1])
+        # disable gauss when gauss is compiled in but asked not to be used
+        if not self.this_gauss_on and "autodisablegauss" in self.extra_opts_supported:
+            cmd += "--maxgaussdepth 0 "
+
+        # note, presimp=0 is braindead for preproc but it's mostly 1 so OK
+        cmd += "--presimp %d " % random.choice([1, 1, 1, 1, 1, 1, 1, 0])
+        cmd += "--confbtwsimp %d " % random.choice([100, 1000])
+
+        if self.dump_red is not None:
+            cmd += "--dumpred %s " % self.dump_red
+            cmd += "--dumpredmaxlen %d " % random.choice([2, 10, 100, 100000])
+            cmd += "--dumpredmaxglue %d " % random.choice([2, 10, 100, 100000])
+
+        if self.only_indep:
+            cmd += "--onlyindep "
+            cmd += "--indep "
+            cmd += ",".join(["%s" % x for x in self.indep_vars]) + " "
+
+        if random.choice([True, False]) and "clid" in self.extra_opts_supported:
+            cmd += "--varsperxorcut %d " % random.randint(4, 6)
+            cmd += "--xorcache %d " % random.choice([0, 1])
+            if random.choice([True, True, True, False]):
+                self.clid_added = True
+                cmd += "--clid "
+            cmd += "--locgmult %.12f " % random.gammavariate(0.5, 0.7)
+            cmd += "--varelimover %d " % random.gammavariate(1, 20)
+            cmd += "--memoutmult %0.12f " % random.gammavariate(0.03, 50)
+            cmd += "--verb %d " % random.choice([0, 0, 0, 0, 1, 2])
+            cmd += "--maple %d " % random.choice([0, 1])
+            if random.randint(0, 2) == 1:
+                cmd += "--reconf %d " % random.choice([3, 4, 6, 7, 12, 13, 14, 15, 16])
+            # cmd += "--undef %d " % random.choice([0, 1])
             cmd += " --reconfat %d " % random.randint(0, 2)
-            cmd += "--burst %d " % random.choice([0, 100, random.randint(0, 10000)])
-            cmd += "--keepguess %s " % random.randint(0, 10)
+            cmd += "--ml  %s " % random.randint(0, 10)
             cmd += "--restart %s " % random.choice(
                 ["geom", "glue", "luby"])
             cmd += "--adjustglue %f " % random.choice([0, 0.5, 0.7, 1.0])
@@ -261,91 +324,76 @@ class Tester:
             cmd += "--otfhyper %s " % random.randint(0, 1)
             # cmd += "--clean %s " % random.choice(["size", "glue", "activity",
             # "prconf"])
-            cmd += "--cacheformoreminim %d " % random.choice([0, 1, 1, 1, 1])
-            cmd += "--stampformoreminim %d " % random.choice([0, 1, 1, 1, 1])
-            cmd += "--alwaysmoremin %s " % random.randint(0, 1)
-            cmd += "--rewardotfsubsume %s " % random.randint(0, 100)
             cmd += "--bothprop %s " % random.randint(0, 1)
             cmd += "--probemaxm %s " % random.choice([0, 10, 100, 1000])
             cmd += "--cachesize %s " % random.randint(10, 100)
             cmd += "--cachecutoff %s " % random.randint(0, 2000)
-            cmd += "--elimstrgy %s " % random.choice(["heuristic", "calculate"])
-            cmd += "--elimcplxupd %s " % random.randint(0, 1)
             cmd += "--occredmax %s " % random.randint(0, 100)
-            cmd += "--noextbinsubs %s " % random.randint(0, 1)
             cmd += "--extscc %s " % random.randint(0, 1)
             cmd += "--distill %s " % random.randint(0, 1)
-            cmd += "--sortwatched %s " % random.randint(0, 1)
             cmd += "--recur %s " % random.randint(0, 1)
             cmd += "--compsfrom %d " % random.randint(0, 2)
             cmd += "--compsvar %d " % random.randint(20000, 500000)
             cmd += "--compslimit %d " % random.randint(0, 3000)
             cmd += "--implicitmanip %s " % random.randint(0, 1)
             cmd += "--occsimp %s " % random.randint(0, 1)
-            cmd += "--occirredmaxmb %s " % random.randint(0, 10)
-            cmd += "--occredmaxmb %s " % random.randint(0, 10)
+            cmd += "--occirredmaxmb %s " % random.gammavariate(0.2, 5)
+            cmd += "--occredmaxmb %s " % random.gammavariate(0.2, 5)
             cmd += "--skipresol %d " % random.choice([1, 1, 1, 0])
             cmd += "--implsubsto %s " % random.choice([0, 10, 1000])
             cmd += "--sync %d " % random.choice([100, 1000, 6000, 100000])
-            cmd += "-m %0.12f " % random.gammavariate(0.4, 2.0)
-            # gammavariate gives us sometimes very low values, sometimes large
+            cmd += "-m %0.12f " % random.gammavariate(0.1, 5.0)
+            cmd += "--maxsccdepth %d " % random.choice([0, 1, 100, 100000])
 
-            if options.sqlite:
+            # more more minim
+            cmd += "--moremoreminim %d " % random.choice([1, 1, 1, 0])
+            cmd += "--moremorecachelimit %d " % int(random.gammavariate(1, 6))
+            cmd += "--moremorestamp %d " % random.choice([1, 1, 1, 0])
+            cmd += "--moremorealways %d " % random.choice([1, 1, 1, 0])
+
+            if self.this_gauss_on:
+                # Reduce iteratively the matrix that is updated
+                cmd += "--iterreduce %s " % random.choice([0, 1])
+
+                # Only run Gaussian Elimination until this depth
+                cmd += "--maxgaussdepth %s " % int(random.gammavariate(1, 20.0))
+
+                # Set maximum no. of rows for gaussian matrix."
+                cmd += "--maxmatrixrows %s " % int(random.gammavariate(5, 15.0))
+
+                # "Automatically disable gauss when performing badly")
+                cmd += "--autodisablegauss %s " % random.choice([0, 1])
+
+                # "Set minimum no. of rows for gaussian matrix.
+                cmd += "--minmatrixrows %s " % int(random.gammavariate(3, 15.0))
+
+                # Save matrix every Nth decision level."
+                cmd += "--savematrix %s " % (int(random.gammavariate(1, 15.0))+1)
+
+                # "Maximum number of matrixes to treat.")
+                cmd += "--maxnummatrixes %s " % int(random.gammavariate(1, 10.0))
+
+            if "sql" in self.extra_opts_supported and random.randint(0, 3) > 0 and self.num_threads == 1 and not self.preproc:
                 cmd += "--sql 2 "
-                cmd += "--sqlrestfull %d " % random.choice([0, 1])
-                cmd += "--sqlresttime %d " % random.choice([0, 1])
+                self.sqlitedbfname = unique_file("fuzz", ".sqlitedb")
+                cmd += "--sqlitedb %s " % self.sqlitedbfname
+                cmd += "--sqlresttime %d " % random.randint(0, 1)
+                cmd += "--cldatadumpratio %0.3f " % random.choice([0.9, 0.1, 0.7])
 
         # the most buggy ones, don't turn them off much, please
         if random.choice([True, False]):
             opts = ["scc", "varelim", "comps", "strengthen", "probe", "intree",
-                    "binpri", "stamp", "cache", "otfsubsume",
+                    "stamp", "cache", "otfsubsume",
                     "renumber", "savemem", "moreminim", "gates", "bva",
-                    "gorshort", "gandrem", "gateeqlit", "schedsimp", "presimp",
-                    "elimcoststrategy"]
+                    "gorshort", "gandrem", "gateeqlit", "schedsimp"]
 
-            opts.extend(self.extra_options_if_supported)
+            if "xor" in self.extra_opts_supported:
+                opts.append("xor")
 
             for opt in opts:
                 cmd += "--%s %d " % (opt, random.randint(0, 1))
 
-            def create_rnd_sched(string_list):
-                opts = string_list.split(",")
-                opts = [a.strip(" ") for a in opts]
-                opts = list(set(opts))
-                if options.verbose:
-                    print("available schedule options: %s" % opts)
-
-                sched = []
-                for i in range(int(random.gammavariate(12, 0.7))):
-                    sched.append(random.choice(opts))
-
-                if "autodisablegauss" in self.extra_options_if_supported and options.test_gauss:
-                    sched.append("occ-gauss")
-
-                return sched
-
-            cmd += self.add_schedule_options(create_rnd_sched, preproc)
-
-        return cmd
-
-    def add_schedule_options(self, create_rnd_sched, preproc):
-        cmd = ""
-
-        sched_opts = "handle-comps,"
-        sched_opts += "scc-vrepl, cache-clean, cache-tryboth,"
-        sched_opts += "sub-impl, intree-probe, probe,"
-        sched_opts += "sub-str-cls-with-bin, distill-cls, scc-vrepl, sub-impl,"
-        sched_opts += "str-impl, cache-clean, sub-str-cls-with-bin, distill-cls, scc-vrepl,"
-        sched_opts += "occ-backw-sub-str, occ-xor, occ-clean-implicit, occ-bve, occ-bva, occ-gates,"
-        sched_opts += "check-cache-size, renumber"
-
-        sched = ",".join(create_rnd_sched(sched_opts))
-        if sched != "" and not preproc:
-            cmd += "--schedule %s " % sched
-
-        sched = ",".join(create_rnd_sched(sched_opts))
-        if sched != "":
-            cmd += "--preschedule %s " % sched
+            cmd += self.rnd_schedule_all(preproc)
 
         return cmd
 
@@ -361,7 +409,7 @@ class Tester:
 
         # construct command
         command = ""
-        if not options.novalgrind and random.randint(0, 10) == 0:
+        if not options.novalgrind and random.randint(1, options.valgrind_freq) == 1:
             command += "valgrind -q --leak-check=full  --error-exitcode=9 "
         command += options.solver
         if rnd_opts is None:
@@ -369,15 +417,16 @@ class Tester:
         command += rnd_opts
         if self.needDebugLib:
             command += "--debuglib %s " % fname
-        if options.verbose is False:
-            command += "--verb 0 "
         command += "--threads %d " % self.num_threads
         command += options.extra_options + " "
         command += fixed_opts + " "
         if fname is not None:
             command += fname
         if fname2:
-            command += " %s --savedstate %s-savedstate.dat " % (fname2, fname2)
+            if self.drat:
+                command += " %s " % fname2
+            else:
+                command += " %s --savedstate %s-savedstate.dat " % (fname2, fname2)
 
         print("Executing: %s " % command)
 
@@ -386,14 +435,16 @@ class Tester:
             print("CPU limit of parent (pid %d)" % os.getpid(), resource.getrlimit(resource.RLIMIT_CPU))
 
         # if need time limit, then limit
-        err_fname = unique_file("%s_err" % fname, ".out")
+        err_fname = unique_file("err", ".out")
         err_file = open(err_fname, "w")
         p = subprocess.Popen(
-            command.rsplit(), stderr=err_file, stdout=subprocess.PIPE, preexec_fn=partial(setlimits, options.maxtime))
+            command.rsplit(), stderr=err_file, stdout=subprocess.PIPE,
+            preexec_fn=partial(setlimits, options.maxtime),
+            universal_newlines=True)
 
         # print time limit after child startup
         if options.verbose:
-            print("CPU limit of parent (pid %d) after startup of child" %
+            print("CPU limit of parent (pid %d) after startup of child: %s secs" %
                   (os.getpid(), resource.getrlimit(resource.RLIMIT_CPU)))
 
         # Get solver output
@@ -404,7 +455,9 @@ class Tester:
             found_something = False
             for line in err_file:
                 print("Error line while executing: %s" % line.strip())
-                if "std::_Ios_Fmtflags" in line or "mzd.h" in line or "lexical_cast.hpp" in line:
+                # don't error out on issues related to UBSAN/ASAN
+                # of clang of other projects
+                if "std::_Ios_Fmtflags" in line or "mzd.h" in line or "lexical_cast.hpp" in line or "MersenneTwister.h" in line:
                     pass
                 else:
                     found_something = True
@@ -413,9 +466,11 @@ class Tester:
                 exit(-1)
 
         os.unlink(err_fname)
+        if self.sqlitedbfname is not None:
+            os.unlink(self.sqlitedbfname)
 
         if options.verbose:
-            print("CPU limit of parent (pid %d) after child finished executing" %
+            print("CPU limit of parent (pid %d) after child finished executing: %s" %
                   (os.getpid(), resource.getrlimit(resource.RLIMIT_CPU)))
 
         return consoleOutput, retcode
@@ -445,6 +500,9 @@ class Tester:
         print("Within time limit: %.2f s" % diff_time)
         print("filename: %s" % fname)
 
+        if options.verbose:
+            print(consoleOutput)
+
         # if library debug is set, check it
         if (self.needDebugLib):
             self.sol_parser.check_debug_lib(checkAgainst)
@@ -465,14 +523,28 @@ class Tester:
             return True
 
         if not unsat:
-            self.sol_parser.test_found_solution(solution, checkAgainst)
+            if len(self.indep_vars) != 0:
+                self.sol_parser.indep_vars_solution_check(fname, self.indep_vars, solution)
+            else:
+                self.sol_parser.test_found_solution(solution, checkAgainst)
+
+            if self.dump_red:
+                self.check_dumped_clauses(fname)
+
             return
 
         # it's UNSAT, let's check with DRAT
         if fname2:
-            toexec = "drat-trim %s %s" % (fname, fname2)
+            toexec = "../../build/tests/drat-trim/drat-trim {cnf} {dratf} {opt}"
+            opt = ""
+            if self.clid_added:
+                opt = "-i "
+            toexec = toexec.format(cnf=fname, dratf=fname2, opt=opt)
             print("Checking DRAT...: ", toexec)
-            p = subprocess.Popen(toexec.rsplit(), stdout=subprocess.PIPE)
+            p = subprocess.Popen(toexec.rsplit(),
+                    stdout=subprocess.PIPE,
+                    universal_newlines=True)
+
             consoleOutput2 = p.communicate()[0]
             diff_time = time.time() - curr_time
 
@@ -506,12 +578,69 @@ class Tester:
             print("Grave bug: SAT-> UNSAT : Other solver found solution!!")
             exit()
 
+    def check_dumped_clauses(self, fname):
+        assert self.dump_red is not None
+
+        tmpfname = unique_file("fuzzTest-dump-test")
+        with open(tmpfname, "w") as tmpf:
+            with open(fname, "r") as x:
+                for line in x:
+                    line = line.strip()
+                    if "c" in line or "p" in line:
+                        continue
+                    tmpf.write(line+"\n")
+
+            with open(self.dump_red, "r") as x:
+                for line in x:
+                    line = line.strip()
+                    tmpf.write(line+"\n")
+
+        print("[dump-check] dump-combined file is: ", tmpfname)
+        if options.verbose:
+            print("dump file is:     ", self.dump_red)
+            print("orig file is:     ",  fname)
+
+        self.old_dump_red = str(self.dump_red)
+        self.dump_red = None
+        self.indep_vars = []
+        self.only_indep = False
+        self.check(tmpfname, checkAgainst=fname)
+
+        os.unlink(tmpfname)
+        os.unlink(self.old_dump_red)
+        print("[dump-check] OK, solution after DUMP has been injected is still OK")
+
     def fuzz_test_one(self):
-        print("\n--- NORMAL TESTING ---")
-        self.num_threads = random.choice([1, 2, 4])
-        self.drat = self.num_threads == 1 and random.choice([True, False])
+        print("--- NORMAL TESTING ---")
+        self.num_threads = random.choice([1, 1, 1, 1, 1, 1, 4])
+        self.num_threads = min(options.max_threads, self.num_threads)
+        self.this_gauss_on = "autodisablegauss" in self.extra_opts_supported and random.choice([True, False, False])
+        if options.gauss:
+            self.this_gauss_on = True
+            assert "autodisablegauss" in self.extra_opts_supported
+
+        self.drat = self.num_threads == 1 and random.randint(0, 10) < 5 and (not self.this_gauss_on)
+        self.sqlitedbfname = None
+        self.preproc = False
+        self.dump_red = random.choice([None, None, None, None, None, True])
+        if self.dump_red is not None:
+            self.dump_red = unique_file("fuzzTest-dump")
+        self.only_indep = random.choice([True, False, False, False, False]) and not self.drat
+
+        if options.only_indep:
+            self.drat = False
+            self.only_indep = True
+
+        if options.only_dump:
+            self.drat = False
+            self.only_indep = False
+            if self.dump_red is None:
+                self.dump_red = unique_file("fuzzTest-dump")
+
         if self.drat:
             fuzzers = fuzzers_drat
+        elif options.gauss:
+            fuzzers = fuzzers_xor
         else:
             fuzzers = fuzzers_nodrat
         fuzzer = random.choice(fuzzers)
@@ -519,17 +648,17 @@ class Tester:
         fname = unique_file("fuzzTest")
         fname_drat = None
         if self.drat:
-            fname_drat = "%s-drat" % fname
+            fname_drat = unique_file("fuzzTest-drat")
 
         # create the fuzz file
         cf = create_fuzz()
         call, todel = cf.create_fuzz_file(fuzzer, fuzzers, fname)
         print("calling %s" % call)
-        status, _ = commands.getstatusoutput(call)
+        status = subprocess.call(call, shell=True)
         if status != 0:
-            fuzzer_call_failed()
+            fuzzer_call_failed(fname)
 
-        if not self.drat:
+        if not self.drat and not self.only_indep and not self.dump_red:
             self.needDebugLib = True
             interspersed_fname = unique_file("fuzzTest")
             seed_for_inters = random.randint(0, 1000000)
@@ -542,14 +671,36 @@ class Tester:
             self.needDebugLib = False
             interspersed_fname = fname
 
+        # calculate indep vars
+        self.indep_vars = []
+        if self.only_indep:
+            max_vars = self.sol_parser.max_vars_in_file(fname)
+            assert max_vars > 0
+
+            self.indep_vars = []
+            myset = {}
+            for _ in range(random.randint(1, 50)):
+                x = random.randint(1, max_vars)
+                if x not in myset:
+                    self.indep_vars.append(x)
+                    myset[x] = 1
+
+            # don't do it for 0-length indep vars
+            if len(self.indep_vars) == 0:
+                self.only_indep = False
+
         self.check(fname=interspersed_fname, fname2=fname_drat)
 
         # remove temporary filenames
         os.unlink(interspersed_fname)
-        for name in todel:
-            os.unlink(name)
         if fname_drat:
             os.unlink(fname_drat)
+        for name in todel:
+            os.unlink(name)
+
+        if self.dump_red is not None:
+            os.unlink(self.dump_red)
+            self.dump_red = None
 
     def delete_file_no_matter_what(self, fname):
         try:
@@ -558,20 +709,26 @@ class Tester:
             pass
 
     def fuzz_test_preproc(self):
-        print("\n--- PREPROC TESTING ---")
+        print("--- PREPROC TESTING ---")
+        self.this_gauss_on = False  # don't do gauss on preproc
         tester.needDebugLib = False
         fuzzer = random.choice(fuzzers_drat)
         self.num_threads = 1
         fname = unique_file("fuzzTest")
         self.drat = False
+        self.preproc = True
+        self.only_indep = False
+        self.indep_vars = []
+        assert self.dump_red is None
+        self.dump_red = None
 
         # create the fuzz file
         cf = create_fuzz()
         call, todel = cf.create_fuzz_file(fuzzer, fuzzers_nodrat, fname)
         print("calling %s : %s" % (fuzzer, call))
-        status, _ = commands.getstatusoutput(call)
+        status = subprocess.call(call, shell=True)
         if status != 0:
-            fuzzer_call_failed()
+            fuzzer_call_failed(fname)
 
         rnd_opts = self.random_options(preproc=True)
 
@@ -592,7 +749,7 @@ class Tester:
                 print("Return code is not 0, error!")
                 exit(-1)
 
-            solution = "%s-solution.txt" % fname
+            solution = "%s-solution.sol" % fname
             ret = self.check(fname=simp, dump_output_fname=solution)
             if ret is not None:
                 # didn't time out, so let's reconstruct the solution
@@ -607,6 +764,7 @@ class Tester:
         os.unlink(fname)
         for name in todel:
             os.unlink(name)
+        assert self.dump_red is None
 
 
 def filter_large_fuzzer(dat):
@@ -622,37 +780,92 @@ def filter_large_fuzzer(dat):
 
     return f
 
-global fuzzers_drat
-global fuzzers_nodrat
-fuzzers_drat = fuzzers_noxor
-fuzzers_nodrat = fuzzers_noxor + fuzzers_xor
-if options.small:
-    fuzzers_drat = filter_large_fuzzer(fuzzers_drat)
-    fuzzers_nodrat = filter_large_fuzzer(fuzzers_nodrat)
 
-print_version()
-tester = Tester()
-tester.needDebugLib = False
-num = 0
-rnd_seed = options.fuzz_seed_start
-if rnd_seed is None:
-    rnd_seed = random.randint(0, 1000*1000*100)
+fuzzers_noxor = [
+    ["../../build/tests/sha1-sat/sha1-gen --nocomment --attack preimage --rounds 20",
+     "--hash-bits", "--seed"],
+    ["../../build/tests/sha1-sat/sha1-gen --nocomment --attack preimage --zero "
+        "--message-bits 400 --rounds 8 --hash-bits 60",
+     "--seed"],
+    # ["build/cnf-fuzz-nossum"],
+    ["../../build/tests/cnf-utils/largefuzzer"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
+    ["../../build/tests/cnf-utils/sgen4 -unsat -n 50", "-s"],
+    ["../../build/tests/cnf-utils//sgen4 -sat -n 50", "-s"],
+    ["../../utils/cnf-utils/cnf-fuzz-brummayer.py", "-s"],
+    ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
+    ["../../utils/cnf-utils/multipart.py", "special"]
+]
+fuzzers_xor = [
+    ["../../utils/cnf-utils/xortester.py", "--seed"],
+    ["../../build/tests/sha1-sat/sha1-gen --xor --attack preimage --rounds 21",
+     "--hash-bits", "--seed"],
+]
 
-while True:
-    toexec = "./fuzz_test.py --fuzzlim 1 --seed %d" % rnd_seed
-    if options.novalgrind:
-        toexec += " --novalgrind"
+
+if __name__ == "__main__":
+    global options
+    global fuzzers_drat
+    global fuzzers_nodrat
+    if not os.path.isdir("out"):
+        print("Directory for outputs, 'out' not present, creating it.")
+        os.mkdir("out")
+
+    # parse options
+    parser = set_up_parser()
+    (options, args) = parser.parse_args()
+    if options.valgrind_freq <= 0:
+        print("Valgrind Frequency must be at least 1")
+        exit(-1)
+
+    fuzzers_drat = fuzzers_noxor
+    fuzzers_nodrat = fuzzers_noxor + fuzzers_xor
     if options.small:
-        toexec += " --small"
+        fuzzers_drat = filter_large_fuzzer(fuzzers_drat)
+        fuzzers_nodrat = filter_large_fuzzer(fuzzers_nodrat)
 
-    print("To re-create fuzz-test below: %s" % toexec)
+    print_version()
+    tester = Tester()
+    tester.needDebugLib = False
+    num = 0
+    rnd_seed = options.fuzz_seed_start
+    if rnd_seed is None:
+        rnd_seed = random.randint(0, 1000*1000*100)
 
-    random.seed(rnd_seed)
-    if random.choice([True, False]):
-        tester.fuzz_test_preproc()
-    else:
-        tester.fuzz_test_one()
-    rnd_seed += 1
-    num += 1
-    if options.fuzz_test_lim is not None and num >= options.fuzz_test_lim:
-        exit(0)
+    while True:
+        toexec = "./fuzz_test.py --fuzzlim 1 --seed %d " % rnd_seed
+        if options.novalgrind:
+            toexec += "--novalgrind "
+        if options.valgrind_freq:
+            toexec += "--valgrindfreq %d " % options.valgrind_freq
+        if options.small:
+            toexec += "--small "
+        if options.gauss:
+            toexec += "--gauss "
+        if options.only_indep:
+            toexec += "--indep "
+        if options.only_dump:
+            toexec += "--dump "
+        toexec += "-m %d " % options.max_threads
+
+        print("")
+        print("")
+        print("--> To re-create fuzz-test below: %s" % toexec)
+
+        random.seed(rnd_seed)
+        if random.randint(0, 10) == 0:
+            tester.fuzz_test_preproc()
+        else:
+            tester.fuzz_test_one()
+        rnd_seed += 1
+        num += 1
+        if options.fuzz_test_lim is not None and num >= options.fuzz_test_lim:
+            exit(0)
