@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 # Copyright (C) 2016  Mate Soos
@@ -30,13 +30,14 @@ import os
 import stat
 import time
 import resource
+import locale
 from functools import partial
 
 
 def unique_file(fname_begin, fname_end=".cnf"):
         counter = 1
         while 1:
-            fname = fname_begin + '_' + str(counter) + fname_end
+            fname = "out/" + fname_begin + '_' + str(counter) + fname_end
             try:
                 fd = os.open(
                     fname, os.O_CREAT | os.O_EXCL, stat.S_IREAD | stat.S_IWRITE)
@@ -46,6 +47,9 @@ def unique_file(fname_begin, fname_end=".cnf"):
                 pass
 
             counter += 1
+            if counter > 300:
+                print("Cannot create unique_file, last try was: %s", fname)
+                exit(-1)
 
 
 def setlimits(maxtime):
@@ -57,7 +61,6 @@ def setlimits(maxtime):
 class solution_parser:
     def __init__(self, options):
         self.options = options
-        pass
 
     @staticmethod
     def test_found_solution(solution, fname, debugLibPart=None):
@@ -91,32 +94,99 @@ class solution_parser:
                 return
 
             # check solution against clause
-            if line[0] != 'c' and line[0] != 'p':
-                if line[0] != 'x':
-                    solution_parser._check_regular_clause(line, solution)
-                else:
-                    assert line[0] == 'x', "Line must start with p, c, v or x"
-                    solution_parser._check_xor_clause(line, solution)
+            try:
+                if line[0] != 'c' and line[0] != 'p':
+                    if line[0] != 'x':
+                        solution_parser._check_regular_clause(line, solution)
+                    else:
+                        assert line[0] == 'x', "Line must start with p, c, v or x"
+                        solution_parser._check_xor_clause(line, solution)
 
                 clauses += 1
+            except:
+                if debugLibPart is not None:
+                    print("--> Error in part: %s. We are reading up to and including part: %s"
+                          % (thisDebugLibPart, debugLibPart-1))
+                raise
 
         f.close()
         print("Verified %d original xor&regular clauses" % clauses)
 
-    def check_unsat(self, fname):
+    def indep_vars_solution_check(self, fname, indep_vars, solution):
+        assert len(indep_vars) > 0
         a = XorToCNF()
         tmpfname = unique_file("tmp_for_xor_to_cnf_convert")
         a.convert(fname, tmpfname)
+
+        with open(tmpfname, "a") as f:
+            # NOTE: the "p cnf..." header will be wrong
+            for i in indep_vars:
+                if i not in solution:
+                    print("ERROR: solution does not contain independent var %d" % i)
+                    print("Independent vars were: %s" % indep_vars)
+                    exit(-1)
+
+                if solution[i]:
+                    f.write("%d 0\n" % i)
+                else:
+                    f.write("-%d 0\n" % i)
+
+        print("-> added partial solution to temporary CNF file %s" % tmpfname)
+
         # execute with the other solver
-        toexec = "lingeling -f %s" % tmpfname
+        toexec = "../../build/tests/minisat/minisat -verb=0 %s" % tmpfname
         print("Solving with other solver: %s" % toexec)
         curr_time = time.time()
         try:
             p = subprocess.Popen(toexec.rsplit(),
                                  stdout=subprocess.PIPE,
-                                 preexec_fn=partial(setlimits, self.options.maxtime))
+                                 preexec_fn=partial(setlimits, self.options.maxtime),
+                                 universal_newlines=True)
         except OSError:
-            print("ERROR: Probably you don't have lingeling installed!")
+            print("ERROR: Minisat didn't run... weird, it's included as a submodule")
+            raise
+
+        consoleOutput2 = p.communicate()[0]
+        os.unlink(tmpfname)
+
+        # if other solver was out of time, then we can't say anything
+        diff_time = time.time() - curr_time
+        if diff_time > self.options.maxtime - self.options.maxtimediff:
+            print("Other solver: too much time to solve, aborted!")
+            return None
+
+        # extract output from the other solver
+        print("Checking other solver output...")
+        otherSolverUNSAT, _, _ = self.parse_solution_from_output(
+            consoleOutput2.split("\n"))
+
+        # check if the other solver finds a solution with the independent vars
+        # set as per partial solution returned
+        if otherSolverUNSAT is True:
+            print("ERROR; The other solver did NOT find a solution with the partial solution given")
+            exit(-1)
+            return False
+
+        print("OK, other solver found a solution using the partial solution")
+
+        return True
+
+    def check_unsat(self, fname):
+        a = XorToCNF()
+        tmpfname = unique_file("tmp_for_xor_to_cnf_convert")
+        a.convert(fname, tmpfname)
+
+        # execute with the other solver
+        toexec = "../../build/tests/minisat/minisat -verb=0 %s" % tmpfname
+        print("Solving with other solver: %s" % toexec)
+        curr_time = time.time()
+        try:
+            p = subprocess.Popen(toexec.rsplit(),
+                                 stdout=subprocess.PIPE,
+                                 preexec_fn=partial(setlimits, self.options.maxtime),
+                                 universal_newlines=True)
+        except OSError:
+            print("ERROR: Minisat didn't run... weird, it's included as a submodule")
             raise
 
         consoleOutput2 = p.communicate()[0]
@@ -190,8 +260,7 @@ class solution_parser:
         if len(output_lines) == 0:
             print("Error! SAT solver output is empty!")
             print("output lines: %s" % output_lines)
-            print("Error code 500")
-            exit(500)
+            exit(-1)
 
         # solution will be put here
         satunsatfound = False
@@ -241,6 +310,15 @@ class solution_parser:
                         break
                     intvar = int(var)
                     solution[abs(intvar)] = (intvar >= 0)
+                continue
+
+            if (line.strip() == ""):
+                continue
+
+            print("Error! SAT solver output contains a line that is neither 'v' nor 'c' nor 's'!")
+            print("Line is:", line.strip())
+            exit(-1)
+
         # print("Parsed values:", solution)
 
         if (ignoreNoSolution is False and
@@ -249,7 +327,7 @@ class solution_parser:
             print("Error: Cannot find line starting with 's' or 'v' in output!")
             print(output_lines)
             print("Error code 500")
-            exit(500)
+            exit(-1)
 
         if (ignoreNoSolution is True and
                 (satunsatfound is False or (
@@ -262,13 +340,13 @@ class solution_parser:
             print("Error: Cannot find if SAT or UNSAT. Maybe didn't finish running?")
             print(output_lines)
             print("Error code 500")
-            exit(500)
+            exit(-1)
 
         if (unsat is False and vlinefound is False):
             print("Error: Solution is SAT, but no 'v' line")
             print (output_lines)
             print("Error code 500")
-            exit(500)
+            exit(-1)
 
         return unsat, solution, conflict
 
@@ -384,10 +462,25 @@ class solution_parser:
 
         return largestPart
 
+    @staticmethod
+    def max_vars_in_file(fname):
+        maxvar = 0
+        with open(fname, "r") as f:
+            for line in f:
+                line = line.strip()
+
+                # ignore comments
+                if not line or line[0] == "c" or line[0] == 'p':
+                    continue
+
+                # calculate max variable
+                maxvar = max(maxvar, get_max_var_from_clause(line))
+
+        return maxvar
+
     @staticmethod
     def _check_regular_clause(line, solution):
         lits = line.split()
-        final = False
         for lit in lits:
             numlit = int(lit)
             if numlit == 0:
@@ -399,6 +492,17 @@ class solution_parser:
             if solution[abs(numlit)] ^ (numlit < 0):
                 return True
 
+        # print not set vars
+        print("Unset vars:")
+        for lit in lits:
+            numlit = int(lit)
+            if numlit == 0:
+                break
+
+            if abs(numlit) not in solution:
+                print("var %d not set" % abs(numlit))
+
+        print("Every other var set to FALSE")
         raise NameError("Error: clause '%s' not satisfied." % line)
 
     @staticmethod
