@@ -33,6 +33,11 @@ THE SOFTWARE.
 using std::thread;
 
 #define CACHE_SIZE 10ULL*1000ULL*1000UL
+#ifndef LIMITMEM
+#define MAX_VARS (1ULL<<28)
+#else
+#define MAX_VARS 3000
+#endif
 
 using namespace CMSat;
 
@@ -60,14 +65,8 @@ namespace CMSat {
             delete log; //this will also close the file
             delete shared_data;
         }
-        CMSatPrivateData(CMSatPrivateData&) //copy should fail
-        {
-            std::exit(-1);
-        }
-        CMSatPrivateData(const CMSatPrivateData&) //copy should fail
-        {
-            std::exit(-1);
-        }
+        CMSatPrivateData(const CMSatPrivateData&) = delete;
+        CMSatPrivateData& operator=(const CMSatPrivateData&) = delete;
 
         vector<Solver*> solvers;
         SharedData *shared_data = NULL;
@@ -81,6 +80,10 @@ namespace CMSat {
         std::ofstream* log = NULL;
         int sql = 0;
         double timeout = std::numeric_limits<double>::max();
+
+        uint64_t previous_sum_conflicts = 0;
+        uint64_t previous_sum_propagations = 0;
+        uint64_t previous_sum_decisions = 0;
     };
 }
 
@@ -119,8 +122,12 @@ DLL_PUBLIC SATSolver::SATSolver(
     data = new CMSatPrivateData(interrupt_asap);
 
     if (config && ((SolverConf*) config)->verbosity) {
-        print_thread_start_and_finish = true;
+        //NOT SAFE
+        //yes -- this system will use a lock, but the solver itself won't(!)
+        //so things will get mangled and printed wrongly
+        //print_thread_start_and_finish = true;
     }
+
     data->solvers.push_back(new Solver((SolverConf*) config, data->must_interrupt));
 }
 
@@ -131,119 +138,227 @@ DLL_PUBLIC SATSolver::~SATSolver()
 
 void update_config(SolverConf& conf, unsigned thread_num)
 {
-    thread_num = thread_num % 20;
-
     //Don't accidentally reconfigure everything to a specific value!
     if (thread_num > 0) {
         conf.reconfigure_val = 0;
     }
+    conf.origSeed += thread_num;
+
+    switch(thread_num % 23) {
+        case 0: {
+            //default setup
+            break;
+        }
 
-    switch(thread_num) {
         case 1: {
             //Minisat-like
+            conf.maple = 0;
             conf.varElimRatioPerIter = 1;
             conf.restartType = Restart::geom;
             conf.polarity_mode = CMSat::PolarityMode::polarmode_neg;
 
-            conf.inc_max_temp_red_cls = 1.02;
+            conf.inc_max_temp_lev2_red_cls = 1.02;
             conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::size)] = 0;
             conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.5;
             break;
         }
         case 2: {
-            //Similar to old CMS except we look at learnt DB size insteead
-            //of conflicts to see if we need to clean.
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::size)] = 0;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0.5;
-            conf.glue_must_keep_clause_if_below_or_eq = 0;
-            conf.inc_max_temp_red_cls = 1.03;
+            conf.maple = 1;
+            conf.modulo_maple_iter = 100;
             break;
         }
         case 3: {
-            conf.max_temporary_learnt_clauses = 40000;
-            conf.var_decay_max = 0.80;
+            //Similar to CMS 2.9 except we look at learnt DB size insteead
+            //of conflicts to see if we need to clean.
+            conf.maple = 0;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0.5;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0;
+            conf.glue_put_lev0_if_below_or_eq = 0;
+            conf.inc_max_temp_lev2_red_cls = 1.03;
             break;
         }
         case 4: {
-            conf.never_stop_search = true;
+            //Similar to CMS 5.0
+            conf.maple = 0;
+            conf.varElimRatioPerIter = 0.4;
+            conf.every_lev1_reduce = 0;
+            conf.every_lev2_reduce = 0;
+            conf.max_temp_lev2_learnt_clauses = 30000;
+            conf.glue_put_lev0_if_below_or_eq = 4;
+
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.5;
             break;
         }
         case 5: {
-            conf.max_temporary_learnt_clauses = 10000;
+            conf.maple = 0;
+            conf.never_stop_search = true;
             break;
         }
         case 6: {
-            conf.do_bva = false;
-            conf.glue_must_keep_clause_if_below_or_eq = 2;
-            conf.varElimRatioPerIter = 1;
-            conf.inc_max_temp_red_cls = 1.04;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0.1;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::size)] = 0.1;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.3;
-            conf.var_decay_max = 0.90; //more 'slow' in adjusting activities
+            //Maple with backtrack
+            conf.maple = 1;
+            conf.modulo_maple_iter = 100;
             break;
         }
-
         case 7: {
-            conf.global_timeout_multiplier = 5;
-            conf.num_conflicts_of_search_inc = 1.15;
-            conf.more_red_minim_limit_cache = 1200;
-            conf.more_red_minim_limit_binary = 600;
-            conf.max_num_lits_more_red_min = 20;
-            conf.max_temporary_learnt_clauses = 10000;
-            conf.var_decay_max = 0.99; //more 'fast' in adjusting activities
+            conf.maple = 0;
+            conf.do_bva = true;
+            conf.glue_put_lev0_if_below_or_eq = 2;
+            conf.varElimRatioPerIter = 1;
+            conf.inc_max_temp_lev2_red_cls = 1.04;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0.1;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.3;
+            conf.var_decay_vsids_max = 0.90; //more 'slow' in adjusting activities
             break;
         }
         case 8: {
             //Different glue limit
-            conf.glue_must_keep_clause_if_below_or_eq = 4;
-            conf.max_num_lits_more_red_min = 3;
-            conf.max_glue_more_minim = 4;
+            conf.maple = 0;
+            conf.glue_put_lev0_if_below_or_eq = 2;
+            conf.glue_put_lev1_if_below_or_eq = 2;
             break;
         }
         case 9: {
-            //Different glue limit
-            conf.glue_must_keep_clause_if_below_or_eq = 2;
+            conf.maple = 0;
+            conf.var_decay_vsids_max = 0.998;
             break;
         }
         case 10: {
+            conf.maple = 0;
+            conf.polarity_mode = CMSat::PolarityMode::polarmode_pos;
+            break;
+        }
+        case 11: {
+            conf.maple = 0;
+            conf.varElimRatioPerIter = 1;
+            conf.restartType = Restart::geom;
+
+            conf.inc_max_temp_lev2_red_cls = 1.01;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.3;
+            break;
+        }
+        case 12: {
+            conf.maple = 0;
+            conf.inc_max_temp_lev2_red_cls = 1.001;
+            break;
+        }
+
+        case 13: {
+            //Minisat-like
+            conf.maple = 1;
+            conf.varElimRatioPerIter = 1;
+            conf.restartType = Restart::geom;
+            conf.polarity_mode = CMSat::PolarityMode::polarmode_neg;
+
+            conf.inc_max_temp_lev2_red_cls = 1.02;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.5;
+            break;
+        }
+        case 14: {
+            //Different glue limit
+            conf.maple = 0;
+            conf.doMinimRedMoreMore = 1;
+            conf.glue_put_lev0_if_below_or_eq = 4;
+            //conf.glue_put_lev2_if_below_or_eq = 8;
+            conf.max_num_lits_more_more_red_min = 3;
+            conf.max_glue_more_minim = 4;
+            break;
+        }
+        case 15: {
+            //Similar to CMS 2.9 except we look at learnt DB size insteead
+            //of conflicts to see if we need to clean.
+            conf.maple = 1;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0.5;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0;
+            conf.glue_put_lev0_if_below_or_eq = 0;
+            conf.inc_max_temp_lev2_red_cls = 1.03;
+            break;
+        }
+        case 16: {
+            //Similar to CMS 5.0
+            conf.maple = 1;
+            conf.varElimRatioPerIter = 0.4;
+            conf.every_lev1_reduce = 0;
+            conf.every_lev2_reduce = 0;
+            conf.max_temp_lev2_learnt_clauses = 30000;
+            conf.glue_put_lev0_if_below_or_eq = 4;
+
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
+            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.5;
+            break;
+        }
+        case 17: {
+            //conf.max_temporary_learnt_clauses = 10000;
+            conf.do_bva = true;
+            break;
+        }
+        case 18: {
+            conf.maple = 0;
+            conf.every_lev1_reduce = 0;
+            conf.every_lev2_reduce = 0;
+            conf.glue_put_lev1_if_below_or_eq = 0;
+            conf.max_temp_lev2_learnt_clauses = 10000;
+            break;
+        }
+
+        case 19: {
+            conf.maple = 1;
+            conf.doMinimRedMoreMore = 1;
+            conf.orig_global_timeout_multiplier = 5;
+            conf.num_conflicts_of_search_inc = 1.15;
+            conf.more_red_minim_limit_cache = 1200;
+            conf.more_red_minim_limit_binary = 600;
+            conf.max_num_lits_more_more_red_min = 20;
+            //conf.max_temporary_learnt_clauses = 10000;
+            conf.var_decay_vsids_max = 0.99; //more 'fast' in adjusting activities
+            break;
+        }
+
+        case 20: {
             //Luby
+            conf.maple = 0;
             conf.restart_inc = 1.5;
             conf.restart_first = 100;
             conf.restartType = CMSat::Restart::luby;
             break;
         }
-        case 11: {
-            conf.glue_must_keep_clause_if_below_or_eq = 3;
-            conf.var_decay_max = 0.97;
-            break;
-        }
-        case 12: {
-            conf.var_decay_max = 0.998;
-            break;
-        }
-        case 13: {
-            conf.polarity_mode = CMSat::PolarityMode::polarmode_pos;
-            break;
-        }
-        case 14: {
-            conf.varElimRatioPerIter = 1;
-            conf.restartType = Restart::geom;
 
-            conf.inc_max_temp_red_cls = 1.01;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::glue)] = 0;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::size)] = 0;
-            conf.ratio_keep_clauses[clean_to_int(ClauseClean::activity)] = 0.3;
+        case 21: {
+            conf.maple = 0;
+            conf.glue_put_lev0_if_below_or_eq = 3;
+            conf.glue_put_lev1_if_below_or_eq = 5;
+            conf.var_decay_vsids_max = 0.97;
             break;
         }
-        case 15: {
-            conf.inc_max_temp_red_cls = 1.001;
+
+        case 22: {
+            conf.maple = 0;
+            conf.doMinimRedMoreMore = 1;
+            conf.orig_global_timeout_multiplier = 5;
+            conf.num_conflicts_of_search_inc = 1.15;
+            conf.more_red_minim_limit_cache = 1200;
+            conf.more_red_minim_limit_binary = 600;
+            conf.max_num_lits_more_more_red_min = 20;
+            //conf.max_temporary_learnt_clauses = 10000;
+            conf.var_decay_vsids_max = 0.99; //more 'fast' in adjusting activities
             break;
         }
 
         default: {
+            conf.maple = ((thread_num % 3) <= 1);
+            conf.modulo_maple_iter = (thread_num % 7)+1;
+            conf.varElimRatioPerIter = 0.1*(thread_num % 9);
+            if (thread_num % 4 == 0) {
+                conf.restartType = Restart::glue;
+            }
+            if (thread_num % 5 == 0) {
+                conf.restartType = Restart::geom;
+            }
+            conf.restart_first = 100 * (0.5*(thread_num % 5));
+            conf.doMinimRedMoreMore = ((thread_num % 5) == 1);
             break;
         }
     }
@@ -259,7 +374,9 @@ DLL_PUBLIC void SATSolver::set_num_threads(unsigned num)
         return;
     }
 
-    if (data->solvers[0]->drat->enabled()) {
+    if (data->solvers[0]->drat->enabled() ||
+        data->solvers[0]->conf.simulate_drat
+    ) {
         std::cerr << "ERROR: DRAT cannot be used in multi-threaded mode" << endl;
         throw std::runtime_error("ERROR: DRAT cannot be used in multi-threaded mode");
     }
@@ -366,12 +483,22 @@ static bool actually_add_clauses_to_threads(CMSatPrivateData* data)
     return ret;
 }
 
+DLL_PUBLIC void SATSolver::set_max_time(double max_time)
+{
+  for (size_t i = 0; i < data->solvers.size(); ++i) {
+    Solver& s = *data->solvers[i];
+    if (max_time >= 0) {
+      s.conf.maxTime = s.get_stats().cpu_time + max_time;
+    }
+  }
+}
+
 DLL_PUBLIC void SATSolver::set_max_confl(int64_t max_confl)
 {
   for (size_t i = 0; i < data->solvers.size(); ++i) {
     Solver& s = *data->solvers[i];
     if (max_confl >= 0) {
-      s.conf.maxConfl = s.get_stats().conflStats.numConflicts + max_confl;
+      s.conf.max_confl = s.get_stats().conflStats.numConflicts + max_confl;
     }
   }
 }
@@ -388,6 +515,7 @@ DLL_PUBLIC void SATSolver::set_no_simplify()
 {
     for (size_t i = 0; i < data->solvers.size(); ++i) {
         Solver& s = *data->solvers[i];
+        s.conf.doRenumberVars = false;
         s.conf.simplify_at_startup = false;
         s.conf.simplify_at_every_startup = false;
         s.conf.full_simplify_at_startup = false;
@@ -399,15 +527,16 @@ DLL_PUBLIC void SATSolver::set_no_simplify()
 DLL_PUBLIC void SATSolver::set_allow_otf_gauss()
 {
     #ifndef USE_GAUSS
-    cout << "ERROR: CryptoMiniSat was not compiled with GAUSS" << endl;
+    std::cerr << "ERROR: CryptoMiniSat was not compiled with GAUSS" << endl;
     exit(-1);
     #else
     for (size_t i = 0; i < data->solvers.size(); ++i) {
         Solver& s = *data->solvers[i];
-        s.conf.reconfigure_at = 0;
-        s.conf.reconfigure_val = 15;
+        //s.conf.reconfigure_at = 0;
+        //s.conf.reconfigure_val = 15;
         s.conf.gaussconf.max_num_matrixes = 10;
         s.conf.gaussconf.autodisable = false;
+        s.conf.allow_elim_xor_vars = false;
     }
     #endif
 }
@@ -436,8 +565,20 @@ DLL_PUBLIC void SATSolver::set_no_bva()
     }
 }
 
+DLL_PUBLIC void SATSolver::set_no_bve()
+{
+    for (size_t i = 0; i < data->solvers.size(); ++i) {
+        Solver& s = *data->solvers[i];
+        s.conf.doVarElim = false;
+    }
+}
+
 DLL_PUBLIC void SATSolver::set_greedy_undef()
 {
+    assert(false && "ERROR: Unfortunately, greedy undef is broken, please don't use it");
+    std::cerr << "ERROR: Unfortunately, greedy undef is broken, please don't use it" << endl;
+    exit(-1);
+
     for (size_t i = 0; i < data->solvers.size(); ++i) {
         Solver& s = *data->solvers[i];
         s.conf.greedy_undef = true;
@@ -455,10 +596,11 @@ DLL_PUBLIC void SATSolver::set_independent_vars(vector<uint32_t>* ind_vars)
 
 DLL_PUBLIC void SATSolver::set_verbosity(unsigned verbosity)
 {
-  for (size_t i = 0; i < data->solvers.size(); ++i) {
-    Solver& s = *data->solvers[i];
+    if (data->solvers.empty())
+        return;
+
+    Solver& s = *data->solvers[0];
     s.conf.verbosity = verbosity;
-  }
 }
 
 DLL_PUBLIC void SATSolver::set_timeout_all_calls(double timeout)
@@ -540,26 +682,32 @@ DLL_PUBLIC bool SATSolver::add_xor_clause(const std::vector<unsigned>& vars, boo
 
 struct OneThreadCalc
 {
-    OneThreadCalc(DataForThread& _data_for_thread, size_t _tid, bool _solve) :
+    OneThreadCalc(
+        DataForThread& _data_for_thread,
+        size_t _tid,
+        bool _solve,
+        bool _only_indep_solution
+    ) :
         data_for_thread(_data_for_thread)
         , tid(_tid)
         , solve(_solve)
+        , only_indep_solution(_only_indep_solution)
     {}
 
     void operator()()
     {
         if (print_thread_start_and_finish) {
             start_time = cpuTime();
-            data_for_thread.update_mutex->lock();
+            //data_for_thread.update_mutex->lock();
             //cout << "c Starting thread " << tid << endl;
-            data_for_thread.update_mutex->unlock();
+            //data_for_thread.update_mutex->unlock();
         }
 
         OneThreadAddCls cls_adder(data_for_thread, tid);
         cls_adder();
         lbool ret;
         if (solve) {
-            ret = data_for_thread.solvers[tid]->solve_with_assumptions(data_for_thread.assumptions);
+            ret = data_for_thread.solvers[tid]->solve_with_assumptions(data_for_thread.assumptions, only_indep_solution);
         } else {
             ret = data_for_thread.solvers[tid]->simplify_with_assumptions(data_for_thread.assumptions);
         }
@@ -567,10 +715,12 @@ struct OneThreadCalc
         if (print_thread_start_and_finish) {
             double end_time = cpuTime();
             data_for_thread.update_mutex->lock();
+            ios::fmtflags f(cout.flags());
             cout << "c Finished thread " << tid << " with result: " << ret
             << " T-diff: " << std::fixed << std::setprecision(2)
             << (end_time-start_time)
             << endl;
+            cout.flags(f);
             data_for_thread.update_mutex->unlock();
         }
 
@@ -589,10 +739,14 @@ struct OneThreadCalc
     const size_t tid;
     double start_time;
     bool solve;
+    bool only_indep_solution;
 };
 
-lbool calc(const vector< Lit >* assumptions, bool solve, CMSatPrivateData *data)
-{
+lbool calc(
+    const vector< Lit >* assumptions,
+    bool solve, CMSatPrivateData *data,
+    bool only_indep_solution = false
+) {
     //Reset the interrupt signal if it was set
     data->must_interrupt->store(false, std::memory_order_relaxed);
 
@@ -627,7 +781,7 @@ lbool calc(const vector< Lit >* assumptions, bool solve, CMSatPrivateData *data)
 
         lbool ret ;
         if (solve) {
-            ret = data->solvers[0]->solve_with_assumptions(assumptions);
+            ret = data->solvers[0]->solve_with_assumptions(assumptions, only_indep_solution);
         } else {
             ret = data->solvers[0]->simplify_with_assumptions(assumptions);
         }
@@ -642,7 +796,7 @@ lbool calc(const vector< Lit >* assumptions, bool solve, CMSatPrivateData *data)
         ; i < data->solvers.size()
         ; i++
     ) {
-        thds.push_back(thread(OneThreadCalc(data_for_thread, i, solve)));
+        thds.push_back(thread(OneThreadCalc(data_for_thread, i, solve, only_indep_solution)));
     }
     for(std::thread& thread : thds){
         thread.join();
@@ -659,13 +813,23 @@ lbool calc(const vector< Lit >* assumptions, bool solve, CMSatPrivateData *data)
     return real_ret;
 }
 
-DLL_PUBLIC lbool SATSolver::solve(const vector< Lit >* assumptions)
+DLL_PUBLIC lbool SATSolver::solve(const vector< Lit >* assumptions, bool only_indep_solution)
 {
-    return calc(assumptions, true, data);
+    //set information data (props, confl, dec)
+    data->previous_sum_conflicts = get_sum_conflicts();
+    data->previous_sum_propagations = get_sum_propagations();
+    data->previous_sum_decisions = get_sum_decisions();
+
+    return calc(assumptions, true, data, only_indep_solution);
 }
 
 DLL_PUBLIC lbool SATSolver::simplify(const vector< Lit >* assumptions)
 {
+    //set information data (props, confl, dec)
+    data->previous_sum_conflicts = get_sum_conflicts();
+    data->previous_sum_propagations = get_sum_propagations();
+    data->previous_sum_decisions = get_sum_decisions();
+
     return calc(assumptions, false, data);
 }
 
@@ -692,11 +856,10 @@ DLL_PUBLIC void SATSolver::new_var()
 
 DLL_PUBLIC void SATSolver::new_vars(const size_t n)
 {
-    if (n >= 1ULL<<28
-        || (data->vars_to_add + n) >= 1ULL<<28
+    if (n >= MAX_VARS
+        || (data->vars_to_add + n) >= MAX_VARS
     ) {
-        cout << "ERROR! Variable requested is far too large" << endl;
-        throw std::runtime_error("ERROR! Variable requested is far too large");
+        throw CMSat::TooManyVarsError();
     }
 
     if (data->log) {
@@ -764,16 +927,6 @@ DLL_PUBLIC void SATSolver::interrupt_asap()
     data->must_interrupt->store(true, std::memory_order_relaxed);
 }
 
-DLL_PUBLIC void SATSolver::open_file_and_dump_irred_clauses(std::string fname) const
-{
-    data->solvers[data->which_solved]->open_file_and_dump_irred_clauses(fname);
-}
-
-void DLL_PUBLIC SATSolver::open_file_and_dump_red_clauses(std::string fname) const
-{
-    data->solvers[data->which_solved]->open_file_and_dump_red_clauses(fname);
-}
-
 void DLL_PUBLIC SATSolver::add_in_partial_solving_stats()
 {
     data->solvers[data->which_solved]->add_in_partial_solving_stats();
@@ -820,6 +973,22 @@ DLL_PUBLIC std::vector<std::pair<Lit, Lit> > SATSolver::get_all_binary_xors() co
     return data->solvers[0]->get_all_binary_xors();
 }
 
+DLL_PUBLIC vector<std::pair<vector<uint32_t>, bool> >
+SATSolver::get_recovered_xors(bool elongate) const
+{
+    vector<std::pair<vector<uint32_t>, bool> > ret;
+    Solver& s = *data->solvers[0];
+
+    std::pair<vector<uint32_t>, bool> tmp;
+    vector<Xor> xors = s.get_recovered_xors(elongate);
+    for(const auto& x: xors) {
+        tmp.first = x.get_vars();
+        tmp.second = x.rhs;
+        ret.push_back(tmp);
+    }
+    return ret;
+}
+
 DLL_PUBLIC void SATSolver::set_sqlite(std::string filename)
 {
     if (data->solvers.size() > 1) {
@@ -832,22 +1001,75 @@ DLL_PUBLIC void SATSolver::set_sqlite(std::string filename)
     data->solvers[0]->set_sqlite(filename);
 }
 
-DLL_PUBLIC void SATSolver::set_mysql(
-    std::string sqlServer
-    , std::string sqlUser
-    , std::string sqlPass
-    , std::string sqlDatabase)
+DLL_PUBLIC uint64_t SATSolver::get_sum_conflicts()
 {
-    if (data->solvers.size() > 1) {
-        std::cerr
-        << "Multithreaded solving and SQL cannot be specified at the same time"
-        << endl;
-        exit(-1);
+    uint64_t conlf = 0;
+    for (size_t i = 0; i < data->solvers.size(); ++i) {
+        Solver& s = *data->solvers[i];
+        conlf += s.sumConflicts;
     }
-    data->sql = 2;
-    data->solvers[0]->set_mysql(
-        sqlServer
-        , sqlUser
-        , sqlPass
-        , sqlDatabase);
+    return conlf;
+}
+
+DLL_PUBLIC uint64_t SATSolver::get_sum_propagations()
+{
+    uint64_t props = 0;
+    for (size_t i = 0; i < data->solvers.size(); ++i) {
+        Solver& s = *data->solvers[i];
+        props += s.sumPropStats.propagations;
+    }
+    return props;
+}
+
+DLL_PUBLIC uint64_t SATSolver::get_sum_decisions()
+{
+    uint64_t dec = 0;
+    for (size_t i = 0; i < data->solvers.size(); ++i) {
+        Solver& s = *data->solvers[i];
+        dec += s.sumSearchStats.decisions;
+    }
+    return dec;
+}
+
+DLL_PUBLIC uint64_t SATSolver::get_last_conflicts()
+{
+    return get_sum_conflicts() - data->previous_sum_conflicts;
+}
+
+DLL_PUBLIC uint64_t SATSolver::get_last_propagations()
+{
+    return get_sum_propagations() - data->previous_sum_propagations;
+}
+
+DLL_PUBLIC uint64_t SATSolver::get_last_decisions()
+{
+    return get_sum_decisions() - data->previous_sum_decisions;
+}
+
+DLL_PUBLIC void SATSolver::open_file_and_dump_irred_clauses(std::string fname) const
+{
+    data->solvers[data->which_solved]->open_file_and_dump_irred_clauses(fname);
+}
+
+void DLL_PUBLIC SATSolver::open_file_and_dump_red_clauses(std::string fname) const
+{
+    data->solvers[data->which_solved]->open_file_and_dump_red_clauses(fname);
+}
+
+void DLL_PUBLIC SATSolver::start_getting_small_clauses(uint32_t max_len, uint32_t max_glue)
+{
+    assert(data->solvers.size() >= 1);
+    data->solvers[0]->start_getting_small_clauses(max_len, max_glue);
+}
+
+bool DLL_PUBLIC SATSolver::get_next_small_clause(std::vector<Lit>& out)
+{
+    assert(data->solvers.size() >= 1);
+    return data->solvers[0]->get_next_small_clause(out);
+}
+
+void DLL_PUBLIC SATSolver::end_getting_small_clauses()
+{
+    assert(data->solvers.size() >= 1);
+    data->solvers[0]->end_getting_small_clauses();
 }
