@@ -35,10 +35,12 @@ THE SOFTWARE.
 #include "implcache.h"
 #include "propengine.h"
 #include "searcher.h"
-#include "cleaningstats.h"
 #include "clauseusagestats.h"
 #include "solvefeatures.h"
 #include "searchstats.h"
+#ifdef CMS_TESTING_ENABLED
+#include "gtest/gtest_prod.h"
+#endif
 
 namespace CMSat {
 
@@ -51,7 +53,7 @@ class ClauseCleaner;
 class Prober;
 class OccSimplifier;
 class SCCFinder;
-class DistillerAllWithAll;
+class DistillerLong;
 class DistillerLongWithImpl;
 class StrImplWImplStamp;
 class CalcDefPolars;
@@ -85,7 +87,7 @@ class Solver : public Searcher
         bool add_clause_outer(const vector<Lit>& lits, bool red = false);
         bool add_xor_clause_outer(const vector<uint32_t>& vars, bool rhs);
 
-        lbool solve_with_assumptions(const vector<Lit>* _assumptions = NULL);
+        lbool solve_with_assumptions(const vector<Lit>* _assumptions, bool only_indep_solution);
         lbool simplify_with_assumptions(const vector<Lit>* _assumptions = NULL);
         void  set_shared_data(SharedData* shared_data);
 
@@ -96,16 +98,22 @@ class Solver : public Searcher
         lbool full_model_value (const uint32_t p) const;  ///<Found model value for var
         const vector<lbool>& get_model() const;
         const vector<Lit>& get_final_conflict() const;
+        vector<pair<Lit, Lit> > get_all_binary_xors() const;
+        vector<Xor> get_recovered_xors(bool elongate);
+
+        //get learnt clauses
+        void start_getting_small_clauses(uint32_t max_len, uint32_t max_glue);
+        bool get_next_small_clause(std::vector<Lit>& out);
+        void end_getting_small_clauses();
 
         void open_file_and_dump_irred_clauses(string fname) const;
         void open_file_and_dump_red_clauses(string fname) const;
-        vector<pair<Lit, Lit> > get_all_binary_xors() const;
 
         static const char* get_version_tag();
         static const char* get_version_sha1();
         static const char* get_compilation_env();
 
-        vector<Lit> get_zero_assigned_lits() const;
+        vector<Lit> get_zero_assigned_lits(const bool backnumber = true, bool only_nvars = false) const;
         void     print_stats(const double cpu_time) const;
         void     print_clause_stats() const;
         size_t get_num_free_vars() const;
@@ -142,7 +150,7 @@ class Solver : public Searcher
         Prober*                prober = NULL;
         InTree*                intree = NULL;
         OccSimplifier*         occsimplifier = NULL;
-        DistillerAllWithAll*   distill_all_with_all = NULL;
+        DistillerLong*         distill_long_cls = NULL;
         DistillerLongWithImpl* dist_long_with_impl = NULL;
         StrImplWImplStamp* dist_impl_with_impl = NULL;
         CompHandler*           compHandler = NULL;
@@ -179,33 +187,28 @@ class Solver : public Searcher
         void attachClause(
             const Clause& c
             , const bool checkAttach = true
-        ) override;
+        );
         void attach_bin_clause(
             const Lit lit1
             , const Lit lit2
             , const bool red
             , const bool checkUnassignedFirst = true
-        ) override;
-        void attach_tri_clause(
-            const Lit lit1
-            , const Lit lit2
-            , const Lit lit3
-            , const bool red
-            , const bool checkUnassignedFirst = true
-        ) override;
-        void detach_tri_clause(
-            Lit lit1
-            , Lit lit2
-            , Lit lit3
-            , bool red
-            , bool allow_empty_watch = false
-        ) override;
+        );
         void detach_bin_clause(
             Lit lit1
             , Lit lit2
             , bool red
             , bool allow_empty_watch = false
-        ) override;
+            , bool allow_change_order = false
+        ) {
+            if (red) {
+                binTri.redBins--;
+            } else {
+                binTri.irredBins--;
+            }
+
+            PropEngine::detach_bin_clause(lit1, lit2, red, allow_empty_watch, allow_change_order);
+        }
         void detachClause(const Clause& c, const bool removeDrat = true);
         void detachClause(const ClOffset offset, const bool removeDrat = true);
         void detach_modified_clause(
@@ -213,7 +216,7 @@ class Solver : public Searcher
             , const Lit lit2
             , const uint32_t origSize
             , const Clause* address
-        ) override;
+        );
         Clause* add_clause_int(
             const vector<Lit>& lits
             , const bool red = false
@@ -223,32 +226,45 @@ class Solver : public Searcher
             , bool addDrat = true
             , const Lit drat_first = lit_Undef
         );
-        template<class T> vector<Lit> clauseBackNumbered(const T& cl) const;
+        template<class T> vector<Lit> clause_outer_numbered(const T& cl) const;
+        template<class T> vector<uint32_t> xor_outer_numbered(const T& cl) const;
         size_t mem_used() const;
         void dump_memory_stats_to_sql();
         void set_sqlite(string filename);
-        void set_mysql(
-            string sqlServer
-            , string sqlUser
-            , string sqlPass
-            , string sqlDatabase);
         //Not Private for testing (maybe could be called from outside)
-        void renumber_variables();
+        bool renumber_variables(bool must_renumber = true);
+        SolveFeatures calculate_features();
+        SolveFeatures last_solve_feature;
 
         uint32_t undefine(vector<uint32_t>& trail_lim_vars);
+        vector<Lit> get_toplevel_units_internal(bool outer_numbering) const;
+
+        #ifdef USE_GAUSS
+        bool init_all_matrixes();
+        #endif
+
 
         //if set to TRUE, a clause has been removed during add_clause_int
         //that contained "lit, ~lit". So "lit" must be set to a value
         //Contains _outer_ variables
-        vector<char> undef_must_set_vars;
+        vector<bool> undef_must_set_vars;
+
+        //Helper
+        void renumber_xors_to_outside(const vector<Xor>& xors, vector<Xor>& xors_ret);
 
     private:
         friend class Prober;
         friend class ClauseDumper;
+        #ifdef CMS_TESTING_ENABLED
+        FRIEND_TEST(SearcherTest, pickpolar_auto_not_changed_by_simp);
+        #endif
+
         lbool iterate_until_solved();
         uint64_t mem_used_vardata() const;
-        SolveFeatures calculate_features() const;
+        void check_reconfigure();
         void reconfigure(int val);
+        bool already_reconfigured = false;
+        long calc_num_confl_to_do_this_iter(const size_t iteration_num) const;
 
         vector<Lit> finalCl_tmp;
         bool sort_and_clean_clause(vector<Lit>& ps, const vector<Lit>& origCl, const bool red);
@@ -256,14 +272,14 @@ class Solver : public Searcher
         vector<std::pair<string, string> > sql_tags;
 
         void check_config_parameters() const;
-        void handle_found_solution(const lbool status);
+        void check_xor_cut_config_sanity() const;
+        void handle_found_solution(const lbool status, const bool only_indep_solution);
         void add_every_combination_xor(const vector<Lit>& lits, bool attach, bool addDrat);
         void add_xor_clause_inter_cleaned_cut(const vector<Lit>& lits, bool attach, bool addDrat);
         unsigned num_bits_set(const size_t x, const unsigned max_size) const;
         void check_too_large_variable_number(const vector<Lit>& lits) const;
         void set_assumptions();
 
-        lbool solve();
         lbool simplify_problem_outside();
         void move_to_outside_assumps(const vector<Lit>* assumps);
         vector<Lit> back_number_from_outside_to_outer_tmp;
@@ -289,13 +305,25 @@ class Solver : public Searcher
         SolveStats solveStats;
         void check_minimization_effectiveness(lbool status);
         void check_recursive_minimization_effectiveness(const lbool status);
-        void extend_solution();
+        void extend_solution(const bool only_indep_solution);
         void check_too_many_low_glues();
         bool adjusted_glue_cutoff_if_too_many = false;
 
         /////////////////////////////
         // Temporary datastructs -- must be cleared before use
         mutable std::vector<Lit> tmpCl;
+        mutable std::vector<uint32_t> tmpXor;
+
+
+        //learnt clause querying
+        uint32_t learnt_clause_query_max_len = std::numeric_limits<uint32_t>::max();
+        uint32_t learnt_clause_query_max_glue = std::numeric_limits<uint32_t>::max();
+        uint32_t learnt_clause_query_at = std::numeric_limits<uint32_t>::max();
+        uint32_t learnt_clause_query_watched_at = std::numeric_limits<uint32_t>::max();
+        uint32_t learnt_clause_query_watched_at_sub = std::numeric_limits<uint32_t>::max();
+        vector<uint32_t> learnt_clause_query_outer_to_without_bva_map;
+        bool all_vars_outside(const vector<Lit>& cl) const;
+        void learnt_clausee_query_map_without_bva(vector<Lit>& cl);
 
         /////////////////////////////
         //Renumberer
@@ -308,8 +336,8 @@ class Solver : public Searcher
             , vector<uint32_t>& interToOuter
         );
         void renumber_clauses(const vector<uint32_t>& outerToInter);
-        void renumber_xor_clauses(const vector<uint32_t>& outerToInter);
         void test_renumbering() const;
+        bool clean_xor_clauses_from_duplicate_and_set_vars();
 
         //Value Unsetter
         struct FindUndef {
@@ -321,15 +349,15 @@ class Solver : public Searcher
             vector<unsigned char> can_be_unset;
             vector<uint32_t>* trail_lim_vars;
             int64_t can_be_unsetSum;
-            bool must_fix;
+            bool must_fix_at_least_one_var;
             uint32_t num_fixed;
             bool verbose = false;
         };
-        FindUndef* undef;
-        bool undef_check_must_fix();
+        FindUndef* undef = NULL;
+        bool undef_must_fix_var();
         void undef_fill_potentials();
         void undef_unset_potentials();
-        template<class C> bool undef_look_at_one_clause(const C c);
+        template<class C> bool undef_clause_surely_satisfied(const C c);
 
 
 
@@ -343,7 +371,6 @@ class Solver : public Searcher
         /////////////////////
         // Data
         size_t               zeroLevAssignsByCNF = 0;
-        size_t               zero_level_assigns_by_searcher = 0;
 
         /////////////////////
         // Clauses
@@ -363,7 +390,7 @@ class Solver : public Searcher
 
 inline void Solver::set_decision_var(const uint32_t var)
 {
-    insertVarOrder(var);
+    insert_var_order_all(var);
 }
 
 inline uint64_t Solver::getNumLongClauses() const
@@ -407,7 +434,7 @@ inline const BinTriStats& Solver::getBinTriStats() const
 }
 
 template<class T>
-inline vector<Lit> Solver::clauseBackNumbered(const T& cl) const
+inline vector<Lit> Solver::clause_outer_numbered(const T& cl) const
 {
     tmpCl.clear();
     for(size_t i = 0; i < cl.size(); i++) {
@@ -417,6 +444,17 @@ inline vector<Lit> Solver::clauseBackNumbered(const T& cl) const
     return tmpCl;
 }
 
+template<class T>
+inline vector<uint32_t> Solver::xor_outer_numbered(const T& cl) const
+{
+    tmpXor.clear();
+    for(size_t i = 0; i < cl.size(); i++) {
+        tmpXor.push_back(map_inter_to_outer(cl[i]));
+    }
+
+    return tmpXor;
+}
+
 inline void Solver::move_to_outside_assumps(const vector<Lit>* assumps)
 {
     outside_assumptions.clear();
@@ -434,13 +472,6 @@ inline void Solver::move_to_outside_assumps(const vector<Lit>* assumps)
     }
 }
 
-inline lbool Solver::solve_with_assumptions(
-    const vector<Lit>* _assumptions
-) {
-    move_to_outside_assumps(_assumptions);
-    return solve();
-}
-
 inline lbool Solver::simplify_with_assumptions(
     const vector<Lit>* _assumptions
 ) {
@@ -535,11 +566,18 @@ inline void Solver::setConf(const SolverConf& _conf)
 
 inline bool Solver::prop_at_head() const
 {
-    return qhead == trail.size();
+    return qhead == trail.size()
+//     #ifdef USE_GAUSS
+//     && gqhead == trail.size()
+//     #endif
+    ;
 }
 
 inline lbool Solver::model_value (const Lit p) const
 {
+    if (model[p.var()] == l_Undef)
+        return l_Undef;
+
     return model[p.var()] ^ p.sign();
 }
 
