@@ -53,6 +53,11 @@ SQLiteStats::~SQLiteStats()
         std::exit(-1);
     }
 
+    ret = sqlite3_finalize(stmtFeat);
+    if (ret != SQLITE_OK) {
+        cout << "Error closing prepared statement" << endl;
+        std::exit(-1);
+    }
     ret = sqlite3_finalize(stmtReduceDB);
     if (ret != SQLITE_OK) {
         cout << "Error closing prepared statement" << endl;
@@ -94,6 +99,7 @@ bool SQLiteStats::setup(const Solver* solver)
     initReduceDBSTMT();
     initTimePassedSTMT();
     initMemUsedSTMT();
+    init_features();
     init_clause_stats_STMT();
 
     return true;
@@ -109,7 +115,8 @@ bool SQLiteStats::connectServer(const int verbosity)
     }
 
     if (sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, NULL)) {
-        cerr << "Problem setting pragma to SQLite DB" << endl;
+        cerr << "ERROR: Problem setting pragma to SQLite DB" << endl;
+        cerr << "c " << sqlite3_errmsg(db) << endl;
         std::exit(-1);
     }
 
@@ -124,9 +131,10 @@ bool SQLiteStats::tryIDInSQL(const Solver* solver)
 {
     std::stringstream ss;
     ss
-    << "INSERT INTO solverRun (runID, `runtime`) values ("
+    << "INSERT INTO solverRun (runID, `runtime`, `gitrev`) values ("
     << runID
     << ", " << time(NULL)
+    << ", '" << solver->get_version_sha1() << "'"
     << ");";
 
     //Inserting element into solverruns to get unique ID
@@ -288,7 +296,7 @@ void SQLiteStats::mem_used(
     //Position
     sqlite3_bind_int64(stmtMemUsed, bindAt++, runID);
     sqlite3_bind_int64(stmtMemUsed, bindAt++, solver->get_solve_stats().numSimplify);
-    sqlite3_bind_int64(stmtMemUsed, bindAt++, solver->sumConflicts());
+    sqlite3_bind_int64(stmtMemUsed, bindAt++, solver->sumConflicts);
     sqlite3_bind_double(stmtMemUsed, bindAt++, given_time);
     //memory stats
     sqlite3_bind_text(stmtMemUsed, bindAt++, name.c_str(), -1, NULL);
@@ -360,7 +368,7 @@ void SQLiteStats::time_passed(
     int bindAt = 1;
     sqlite3_bind_int64(stmtTimePassed, bindAt++, runID);
     sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->get_solve_stats().numSimplify);
-    sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->sumConflicts());
+    sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->sumConflicts);
     sqlite3_bind_double(stmtTimePassed, bindAt++, cpuTime());
     sqlite3_bind_text(stmtTimePassed, bindAt++, name.c_str(), -1, NULL);
     sqlite3_bind_double(stmtTimePassed, bindAt++, time_passed);
@@ -397,7 +405,7 @@ void SQLiteStats::time_passed_min(
     int bindAt = 1;
     sqlite3_bind_int64(stmtTimePassed, bindAt++, runID);
     sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->get_solve_stats().numSimplify);
-    sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->sumConflicts());
+    sqlite3_bind_int64(stmtTimePassed, bindAt++, solver->sumConflicts);
     sqlite3_bind_double(stmtTimePassed, bindAt++, cpuTime());
     sqlite3_bind_text(stmtTimePassed, bindAt++, name.c_str(), -1, NULL);
     sqlite3_bind_double(stmtTimePassed, bindAt++, time_passed);
@@ -426,23 +434,130 @@ void SQLiteStats::time_passed_min(
     }
 }
 
+void SQLiteStats::init_features() {
+    const size_t numElems = 67;
+
+    std::stringstream ss;
+    ss << "insert into `features`"
+    << "("
+    //Position
+    << "  `runID`, `simplifications`, `restarts`, `conflicts`, `latest_feature_calc`"
+
+    //Base data
+    << ", `numVars`"
+    << ", `numClauses`"
+    << ", `var_cl_ratio`"
+
+    //Clause distribution
+    << ", `binary`"
+    << ", `horn`"
+    << ", `horn_mean`"
+    << ", `horn_std`"
+    << ", `horn_min`"
+    << ", `horn_max`"
+    << ", `horn_spread`"
+//14
+    << ", `vcg_var_mean`"
+    << ", `vcg_var_std`"
+    << ", `vcg_var_min`"
+    << ", `vcg_var_max`"
+    << ", `vcg_var_spread`"
+
+    << ", `vcg_cls_mean`"
+    << ", `vcg_cls_std`"
+    << ", `vcg_cls_min`"
+    << ", `vcg_cls_max`"
+    << ", `vcg_cls_spread`"
+
+    << ", `pnr_var_mean`"
+    << ", `pnr_var_std`"
+    << ", `pnr_var_min`"
+    << ", `pnr_var_max`"
+    << ", `pnr_var_spread`"
+
+    << ", `pnr_cls_mean`"
+    << ", `pnr_cls_std`"
+    << ", `pnr_cls_min`"
+    << ", `pnr_cls_max`"
+    << ", `pnr_cls_spread`"
+//34
+    //Conflict clauses
+    << ", `avg_confl_size`"
+    << ", `confl_size_min`"
+    << ", `confl_size_max`"
+    << ", `avg_confl_glue`"
+    << ", `confl_glue_min`"
+    << ", `confl_glue_max`"
+    << ", `avg_num_resolutions`"
+    << ", `num_resolutions_min`"
+    << ", `num_resolutions_max`"
+    << ", `learnt_bins_per_confl`"
+//44
+    //Search
+    << ", `avg_branch_depth`"
+    << ", `branch_depth_min`"
+    << ", `branch_depth_max`"
+    << ", `avg_trail_depth_delta`"
+    << ", `trail_depth_delta_min`"
+    << ", `trail_depth_delta_max`"
+    << ", `avg_branch_depth_delta`"
+    << ", `props_per_confl`"
+    << ", `confl_per_restart`"
+    << ", `decisions_per_conflict`"
+//54
+    //clause distributions
+    << ", `red_glue_distr_mean`"
+    << ", `red_glue_distr_var`"
+    << ", `red_size_distr_mean`"
+    << ", `red_size_distr_var`"
+    << ", `red_activity_distr_mean`"
+    << ", `red_activity_distr_var`"
+//60
+    << ", `irred_glue_distr_mean`"
+    << ", `irred_glue_distr_var`"
+    << ", `irred_size_distr_mean`"
+    << ", `irred_size_distr_var`"
+    << ", `irred_activity_distr_mean`"
+    << ", `irred_activity_distr_var`"
+//66
+    << ") values ";
+    writeQuestionMarks(
+        numElems
+        , ss
+    );
+    ss << ";";
+
+    //Prepare the statement
+    if (sqlite3_prepare(db, ss.str().c_str(), -1, &stmtFeat, NULL)) {
+        cerr << "ERROR in sqlite_stmt_prepare(), INSERT failed"
+        << endl
+        << sqlite3_errmsg(db)
+        << endl
+        << "Query was: " << ss.str()
+        << endl;
+        std::exit(-1);
+    }
+}
+
 //Prepare statement for restart
 void SQLiteStats::initRestartSTMT()
 {
-    const size_t numElems = 74;
+    const size_t numElems = 67;
 
     std::stringstream ss;
     ss << "insert into `restart`"
     << "("
     //Position
-    << "  `runID`, `simplifications`, `restarts`, `conflicts`, `runtime`"
+    << "  `runID`, `simplifications`, `restarts`, `conflicts`, `latest_feature_calc`"
+    << ", `runtime` "
 
     //Clause stats
-    << ", numIrredBins, numIrredTris, numIrredLongs"
-    << ", numRedBins, numRedTris, numRedLongs"
+    << ", numIrredBins, numIrredLongs"
+    << ", numRedBins, numRedLongs"
     << ", numIrredLits, numRedLits"
 
     //Conflict stats
+    << ", `restart_type`"
     << ", `glue`, `glueSD`, `glueMin`, `glueMax`"
     << ", `size`, `sizeSD`, `sizeMin`, `sizeMax`"
     << ", `resolutions`, `resolutionsSD`, `resolutionsMin`, `resolutionsMax`"
@@ -455,19 +570,17 @@ void SQLiteStats::initRestartSTMT()
 
     //Propagations
     << ", `propBinIrred` , `propBinRed` "
-    << ", `propTriIrred` , `propTriRed`"
     << ", `propLongIrred` , `propLongRed`"
 
     //Conflicts
     << ", `conflBinIrred`, `conflBinRed`"
-    << ", `conflTriIrred`, `conflTriRed`"
     << ", `conflLongIrred`, `conflLongRed`"
 
     //Reds
-    << ", `learntUnits`, `learntBins`, `learntTris`, `learntLongs`"
+    << ", `learntUnits`, `learntBins`, `learntLongs`"
 
     //Resolutions
-    << ", `resolBinIrred`, `resolBinRed`, `resolTriIrred`, `resolTriRed`, `resolLIrred`, `resolLRed`"
+    << ", `resolBinIrred`, `resolBinRed`, `resolLIrred`, `resolLRed`"
 
     //Var stats
     << ", `propagations`"
@@ -494,8 +607,119 @@ void SQLiteStats::initRestartSTMT()
     }
 }
 
+void SQLiteStats::features(
+    const Solver* solver
+    , const Searcher* search
+    , const SolveFeatures& feat
+) {
+    int bindAt = 1;
+    sqlite3_bind_int64(stmtFeat, bindAt++, runID);
+    sqlite3_bind_int64(stmtFeat, bindAt++, solver->get_solve_stats().numSimplify);
+    sqlite3_bind_int64(stmtFeat, bindAt++, search->sumRestarts());
+    sqlite3_bind_int64(stmtFeat, bindAt++, solver->sumConflicts);
+    sqlite3_bind_int(stmtFeat, bindAt++, solver->latest_feature_calc);
+
+    sqlite3_bind_int64(stmtFeat, bindAt++, feat.numVars);
+    sqlite3_bind_int64(stmtFeat, bindAt++, feat.numClauses);
+    sqlite3_bind_int64(stmtFeat, bindAt++, feat.var_cl_ratio);
+
+    //Clause distribution
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.binary);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn_std);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.horn_spread);
+
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_var_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_var_std);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_var_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_var_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_var_spread);
+
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_cls_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_cls_std);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_cls_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_cls_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.vcg_cls_spread);
+
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_var_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_var_std);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_var_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_var_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_var_spread);
+
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_cls_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_cls_std);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_cls_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_cls_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.pnr_cls_spread);
+
+    //Conflict clauses
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_confl_size);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.confl_size_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.confl_size_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_confl_glue);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.confl_glue_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.confl_glue_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_num_resolutions);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.num_resolutions_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.num_resolutions_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.learnt_bins_per_confl);
+
+    //Search
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_branch_depth);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.branch_depth_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.branch_depth_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_trail_depth_delta);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.trail_depth_delta_min);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.trail_depth_delta_max);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.avg_branch_depth_delta);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.props_per_confl);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.confl_per_restart);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.decisions_per_conflict);
+
+    //red stats
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.glue_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.glue_distr_var);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.size_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.size_distr_var);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.activity_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.red_cl_distrib.activity_distr_var);
+
+    //irred stats
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.glue_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.glue_distr_var);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.size_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.size_distr_var);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.activity_distr_mean);
+    sqlite3_bind_double(stmtFeat, bindAt++, feat.irred_cl_distrib.activity_distr_var);
+
+    int rc = sqlite3_step(stmtFeat);
+    if (rc != SQLITE_DONE) {
+        cerr << "ERROR  while executing restart insertion SQLite prepared statement"
+        << endl
+        << "Error from sqlite: "
+        << sqlite3_errmsg(db)
+        << endl;
+
+        std::exit(-1);
+    }
+
+    if (sqlite3_reset(stmtFeat)) {
+        cerr << "Error calling sqlite3_reset on stmtFeat" << endl;
+        std::exit(-1);
+    }
+    if (sqlite3_clear_bindings(stmtFeat)) {
+        cerr << "Error calling sqlite3_clear_bindings on stmtFeat" << endl;
+        std::exit(-1);
+    }
+}
+
 void SQLiteStats::restart(
-    const PropStats& thisPropStats
+    const std::string& restart_type
+    , const PropStats& thisPropStats
     , const SearchStats& thisStats
     , const Solver* solver
     , const Searcher* search
@@ -507,22 +731,22 @@ void SQLiteStats::restart(
     sqlite3_bind_int64(stmtRst, bindAt++, runID);
     sqlite3_bind_int64(stmtRst, bindAt++, solver->get_solve_stats().numSimplify);
     sqlite3_bind_int64(stmtRst, bindAt++, search->sumRestarts());
-    sqlite3_bind_int64(stmtRst, bindAt++, solver->sumConflicts());
+    sqlite3_bind_int64(stmtRst, bindAt++, solver->sumConflicts);
+    sqlite3_bind_int(stmtRst, bindAt++, solver->latest_feature_calc);
     sqlite3_bind_double(stmtRst, bindAt++, cpuTime());
 
 
     sqlite3_bind_int64(stmtRst, bindAt++, binTri.irredBins);
-    sqlite3_bind_int64(stmtRst, bindAt++, binTri.irredTris);
     sqlite3_bind_int64(stmtRst, bindAt++, solver->get_num_long_irred_cls());
 
     sqlite3_bind_int64(stmtRst, bindAt++, binTri.redBins);
-    sqlite3_bind_int64(stmtRst, bindAt++, binTri.redTris);
     sqlite3_bind_int64(stmtRst, bindAt++, solver->get_num_long_red_cls());
 
     sqlite3_bind_int64(stmtRst, bindAt++, solver->litStats.irredLits);
     sqlite3_bind_int64(stmtRst, bindAt++, solver->litStats.redLits);
 
     //Conflict stats
+    sqlite3_bind_text(stmtRst, bindAt++, restart_type.c_str(), -1, NULL);
     sqlite3_bind_double(stmtRst, bindAt++, searchHist.glueHist.getLongtTerm().avg());
     sqlite3_bind_double(stmtRst, bindAt++, std:: sqrt(searchHist.glueHist.getLongtTerm().var()));
     sqlite3_bind_double(stmtRst, bindAt++, searchHist.glueHist.getLongtTerm().getMin());
@@ -562,30 +786,23 @@ void SQLiteStats::restart(
     //Prop
     sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsBinIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsBinRed);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsTriIrred);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsTriRed);
     sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsLongIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisPropStats.propsLongRed);
 
     //Confl
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsBinIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsBinRed);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsTriIrred);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsTriRed);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsLongIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.conflStats.conflsLongRed);
 
     //Red
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.learntUnits);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.learntBins);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisStats.learntTris);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.learntLongs);
 
     //Resolv stats
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.binIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.binRed);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.triIrred);
-    sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.triRed);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.longIrred);
     sqlite3_bind_int64(stmtRst, bindAt++, thisStats.resolvs.longRed);
 
@@ -631,35 +848,29 @@ void SQLiteStats::restart(
 //Prepare statement for restart
 void SQLiteStats::initReduceDBSTMT()
 {
-    const size_t numElems = 40;
+    const size_t numElems = 19;
 
     std::stringstream ss;
     ss << "insert into `reduceDB`"
     << "("
     //Position
     << "  `runID`, `simplifications`, `restarts`, `conflicts`, `runtime`"
-    << ", `reduceDBs`"
 
-    //Actual data
-    << ", `irredClsVisited`, `irredLitsVisited`"
-    << ", `redClsVisited`, `redLitsVisited`"
-
-    //Clean data
-    << ", removedNum, removedLits, removedGlue"
-    << ", removedResolBinIrred, removedResolBinRed"
-    << ", removedResolTriIrred, removedResolTriRed"
-    << ", removedResolLIrred, removedResolLRed"
-    << ", removedAge"
-    << ", removedLitVisited, removedProp, removedConfl"
-    << ", removedLookedAt, removedUsedUIP"
-
-    << ", remainNum, remainLits, remainGlue"
-    << ", remainResolBinIrred, remainResolBinRed"
-    << ", remainResolTriIrred, remainResolTriRed"
-    << ", remainResolLIrred, remainResolLRed"
-    << ", remainAge"
-    << ", remainLitVisited, remainProp, remainConfl"
-    << ", remainLookedAt, remainUsedUIP"
+    //data
+    << ", `clauseID`"
+    << ", `dump_no`"
+    << ", `conflicts_made`"
+    << ", `sum_of_branch_depth_conflict`"
+    << ", `propagations_made`"
+    << ", `clause_looked_at`"
+    << ", `used_for_uip_creation`"
+    << ", `last_touched_diff`"
+    << ", `activity_rel`"
+    << ", `locked`"
+    << ", `in_xor`"
+    << ", `glue`"
+    << ", `size`"
+    << ", `ttl`"
     << ") values ";
     writeQuestionMarks(
         numElems
@@ -682,69 +893,42 @@ void SQLiteStats::initReduceDBSTMT()
 }
 
 void SQLiteStats::reduceDB(
-    const ClauseUsageStats& irredStats
-    , const ClauseUsageStats& redStats
-    , const CleaningStats& clean
-    , const Solver* solver
+    const Solver* solver
+    , const bool locked
+    , const Clause* cl
 ) {
+    assert(cl->stats.dump_number != std::numeric_limits<uint32_t>::max());
 
     int bindAt = 1;
     sqlite3_bind_int64(stmtReduceDB, bindAt++, runID);
     sqlite3_bind_int64(stmtReduceDB, bindAt++, solver->get_solve_stats().numSimplify);
     sqlite3_bind_int64(stmtReduceDB, bindAt++, solver->sumRestarts());
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, solver->sumConflicts());
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, solver->sumConflicts);
     sqlite3_bind_double(stmtReduceDB, bindAt++, cpuTime());
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, solver->reduceDB->get_nbReduceDB());
 
-    //Clause data for IRRED
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, irredStats.sumLookedAt);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, 0);
+    //data
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.ID);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.dump_number);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.conflicts_made);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.sum_of_branch_depth_conflict);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.propagations_made);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.clause_looked_at);
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, cl->stats.used_for_uip_creation);
 
-    //Clause data for RED
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, redStats.sumLookedAt);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, 0);
+    uint64_t last_touched_diff;
+    if (cl->stats.last_touched == 0) {
+        last_touched_diff = solver->sumConflicts-cl->stats.introduced_at_conflict;
+    } else {
+        last_touched_diff = solver->sumConflicts-cl->stats.last_touched;
+    }
+    sqlite3_bind_int64(stmtReduceDB, bindAt++, last_touched_diff);
 
-    //removed
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.num);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.lits);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.glue);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.binIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.binRed);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.triIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.triRed);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.longIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.antec_data.longRed);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.age);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, 0);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.numProp);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.numConfl);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.numLookedAt);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.removed.used_for_uip_creation);
-
-    //remain
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.num);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.lits);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.glue);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.binIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.binRed);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.triIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.triRed);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.longIrred);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.antec_data.longRed);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.age);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, 0);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.numProp);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.numConfl);
-
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.numLookedAt);
-    sqlite3_bind_int64(stmtReduceDB, bindAt++, clean.remain.used_for_uip_creation);
+    sqlite3_bind_double(stmtReduceDB, bindAt++, (double)cl->stats.activity/(double)solver->get_cla_inc());
+    sqlite3_bind_int(stmtReduceDB, bindAt++, locked);
+    sqlite3_bind_int(stmtReduceDB, bindAt++, cl->used_in_xor());
+    sqlite3_bind_int(stmtReduceDB, bindAt++, cl->stats.glue);
+    sqlite3_bind_int(stmtReduceDB, bindAt++, cl->size());
+    sqlite3_bind_int(stmtReduceDB, bindAt++, cl->stats.ttl);
 
     int rc = sqlite3_step(stmtReduceDB);
     if (rc != SQLITE_DONE) {
@@ -771,7 +955,7 @@ void SQLiteStats::reduceDB(
 
 void SQLiteStats::init_clause_stats_STMT()
 {
-    const size_t numElems = 47;
+    const size_t numElems = 67;
 
     std::stringstream ss;
     ss << "insert into `clauseStats`"
@@ -779,7 +963,9 @@ void SQLiteStats::init_clause_stats_STMT()
     << " `runID`,"
     << " `simplifications`,"
     << " `restarts`,"
+    << " `prev_restart`,"
     << " `conflicts`,"
+    << " `latest_feature_calc`,"
     << " `clauseID`,"
     << ""
     << " `glue`,"
@@ -787,21 +973,29 @@ void SQLiteStats::init_clause_stats_STMT()
     << " `conflicts_this_restart`,"
     << " `num_overlap_literals`,"
     << " `num_antecedents`,"
+    << " `num_total_lits_antecedents`,"
     << " `antecedents_avg_size`,"
 
+    << " `last_dec_var_act_vsids_0`,"
+    << " `last_dec_var_act_vsids_1`,"
+    << " `first_dec_var_act_vsids_0`,"
+    << " `first_dec_var_act_vsids_1`,"
+
     << " `backtrack_level`,"
     << " `decision_level`,"
+    << " `decision_level_pre1`,"
+    << " `decision_level_pre2`,"
     << " `trail_depth_level`,"
+    << " `cur_restart_type` ,"
 
     << " `atedecents_binIrred`,"
     << " `atedecents_binRed`,"
-    << " `atedecents_triIrred`,"
-    << " `atedecents_triRed`,"
     << " `atedecents_longIrred`,"
     << " `atedecents_longRed`,"
 
     << " `vsids_vars_avg`,"
     << " `vsids_vars_var`,"
+    << " `vsids_vars_min`,"
     << " `vsids_vars_max`,"
 
     << " `antecedents_glue_long_reds_avg`,"
@@ -827,12 +1021,23 @@ void SQLiteStats::init_clause_stats_STMT()
     << " `antecedents_antecedents_vsids_avg`,"
 
     << " `decision_level_hist`,"
-    << " `backtrack_level_hist`,"
+    << " `backtrack_level_hist_lt`,"
     << " `trail_depth_level_hist`,"
     << " `vsids_vars_hist`,"
     << " `size_hist`,"
     << " `glue_hist`,"
-    << " `num_antecedents_hist`"
+    << " `num_antecedents_hist`,"
+    << " `antec_sum_size_hist`,"
+    << " `antec_overlap_hist`,"
+    << " `branch_depth_hist_queue`,"
+    << " `trail_depth_hist`,"
+    << " `trail_depth_hist_longer`,"
+    << " `num_resolutions_hist`,"
+    << " `confl_size_hist`,"
+    << " `trail_depth_delta_hist`,"
+    << " `backtrack_level_hist`,"
+    << " `glue_hist_queue`,"
+    << " `glue_hist_long`"
     << ") values ";
     writeQuestionMarks(
         numElems
@@ -864,7 +1069,12 @@ void SQLiteStats::dump_clause_stats(
     , size_t decision_level
     , size_t trail_depth
     , uint64_t conflicts_this_restart
+    , const std::string& restart_type
     , const SearchHist& hist
+    , const double last_dec_var_act_vsids_0
+    , const double last_dec_var_act_vsids_1
+    , const double first_dec_var_act_vsids_0
+    , const double first_dec_var_act_vsids_1
 ) {
     uint32_t num_overlap_literals = antec_data.sum_size()-(antec_data.num()-1)-size;
 
@@ -872,7 +1082,13 @@ void SQLiteStats::dump_clause_stats(
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, runID);
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, solver->get_solve_stats().numSimplify);
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, solver->sumRestarts());
-    sqlite3_bind_int64(stmt_clause_stats, bindAt++, solver->sumConflicts());
+    if (solver->sumRestarts() == 0) {
+        sqlite3_bind_int64(stmt_clause_stats, bindAt++, 0);
+    } else {
+        sqlite3_bind_int64(stmt_clause_stats, bindAt++, solver->sumRestarts()-1);
+    }
+    sqlite3_bind_int64(stmt_clause_stats, bindAt++, solver->sumConflicts);
+    sqlite3_bind_int(stmt_clause_stats, bindAt++, solver->latest_feature_calc);
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, clauseID);
 
     sqlite3_bind_int(stmt_clause_stats, bindAt++, glue);
@@ -880,16 +1096,22 @@ void SQLiteStats::dump_clause_stats(
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, conflicts_this_restart);
     sqlite3_bind_int(stmt_clause_stats, bindAt++, num_overlap_literals);
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.num());
+    sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.sum_size());
     sqlite3_bind_double(stmt_clause_stats, bindAt++, (double)antec_data.sum_size()/(double)antec_data.num() );
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, last_dec_var_act_vsids_0);
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, last_dec_var_act_vsids_1);
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, first_dec_var_act_vsids_0);
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, first_dec_var_act_vsids_1);
 
     sqlite3_bind_int(stmt_clause_stats, bindAt++, backtrack_level);
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, decision_level);
+    sqlite3_bind_int64(stmt_clause_stats, bindAt++, hist.branchDepthHistQueue.prev(1));
+    sqlite3_bind_int64(stmt_clause_stats, bindAt++, hist.branchDepthHistQueue.prev(2));
     sqlite3_bind_int64(stmt_clause_stats, bindAt++, trail_depth);
+    sqlite3_bind_text(stmt_clause_stats, bindAt++,  restart_type.c_str(), -1, NULL);
 
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.binIrred);
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.binRed);
-    sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.triIrred);
-    sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.triRed);
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.longIrred);
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.longRed);
 
@@ -901,6 +1123,7 @@ void SQLiteStats::dump_clause_stats(
     sqlite3_bind_double(stmt_clause_stats, bindAt++, antec_data.glue_long_reds.avg());
     sqlite3_bind_double(stmt_clause_stats, bindAt++, antec_data.glue_long_reds.var());
     sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.glue_long_reds.getMin());
+    sqlite3_bind_int(stmt_clause_stats, bindAt++, antec_data.glue_long_reds.getMax());
 
     sqlite3_bind_double(stmt_clause_stats, bindAt++, antec_data.age_long_reds.avg() );
     sqlite3_bind_double(stmt_clause_stats, bindAt++, antec_data.age_long_reds.var() );
@@ -924,9 +1147,22 @@ void SQLiteStats::dump_clause_stats(
     sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.trailDepthHistLT.avg());
     sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.vsidsVarsAvgLT.avg());
     sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.conflSizeHistLT.avg());
-    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.glueHistLT.avg());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.glueHistLTAll.avg());
     sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.numResolutionsHistLT.avg());
 
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.antec_data_sum_sizeHistLT.avg());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.overlapHistLT.avg());
+
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.branchDepthHistQueue.avg_nocheck());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.trailDepthHist.avg_nocheck());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.trailDepthHistLonger.avg_nocheck());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.numResolutionsHist.avg());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.conflSizeHist.avg());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.trailDepthDeltaHist.avg());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.backtrackLevelHist.avg_nocheck());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.glueHist.avg_nocheck());
+    sqlite3_bind_double(stmt_clause_stats, bindAt++, hist.glueHist.getLongtTerm().avg());
+
     int rc = sqlite3_step(stmt_clause_stats);
     if (rc != SQLITE_DONE) {
         cout
