@@ -84,7 +84,7 @@ void VarReplacer::check_no_replaced_var_set() const
 void VarReplacer::new_vars(const size_t n)
 {
     size_t oldsize = table.size();
-    table.resize(table.size()+n, lit_Undef);
+    table.insert(table.end(), n, lit_Undef);
     for(size_t i = oldsize; i < table.size(); i++) {
         table[i] = Lit(i, false);
     }
@@ -148,10 +148,16 @@ void VarReplacer::update_vardata_and_activities(
     assert(solver->varData[replaced_with].removed == Removed::none);
     assert(solver->value(replaced_with) == l_Undef);
 
-    double orig_act = solver->activ_glue[orig];
-    double repl_with_act = solver->activ_glue[replaced_with];
-    if (orig_act + repl_with_act >= orig_act) {
-        solver->activ_glue[replaced_with] += orig_act;
+    double orig_act_vsids = solver->var_act_vsids[orig];
+    double repl_with_act_vsids = solver->var_act_vsids[replaced_with];
+    if (orig_act_vsids + repl_with_act_vsids >= orig_act_vsids) {
+        solver->var_act_vsids[replaced_with] += orig_act_vsids;
+    }
+
+    double repl_with_act_maple = solver->var_act_maple[replaced_with];
+    double orig_act_maple = solver->var_act_maple[orig];
+    if (orig_act_maple + repl_with_act_maple >= orig_act_maple) {
+        solver->var_act_maple[replaced_with] += orig_act_maple;
     }
 }
 
@@ -175,13 +181,13 @@ bool VarReplacer::enqueueDelayedEnqueue()
         return false;
 
     solver->ok = solver->propagate<false>().isNULL();
-    return solver->ok;
+    return solver->okay();
 }
 
 void VarReplacer::attach_delayed_attach()
 {
     for(Clause* c: delayed_attach_or_free) {
-        if (c->size() <= 3) {
+        if (c->size() <= 2) {
             solver->cl_alloc.clauseFree(c);
         } else {
             c->unset_removed();
@@ -216,7 +222,9 @@ bool VarReplacer::perform_replace()
     const size_t origTrailSize = solver->trail_size();
 
     solver->clauseCleaner->remove_and_clean_all();
+    #ifdef DEBUG_ATTACH_MORE
     solver->test_all_clause_attached();
+    #endif
 
     //Printing stats
     if (solver->conf.verbosity >= 5)
@@ -228,7 +236,9 @@ bool VarReplacer::perform_replace()
     runStats.actuallyReplacedVars = replacedVars -lastReplacedVars;
     lastReplacedVars = replacedVars;
 
+    #ifdef DEBUG_ATTACH_MORE
     solver->test_all_clause_attached();
+    #endif
     assert(solver->prop_at_head());
 
     #ifdef DEBUG_IMPLICIT_STATS
@@ -254,6 +264,9 @@ bool VarReplacer::perform_replace()
     }
     solver->clean_occur_from_removed_clauses_only_smudged();
     attach_delayed_attach();
+    if (!replace_xor_clauses()) {
+        goto end;
+    }
 
     //While replacing the clauses
     //we cannot(for implicits) and/or shouldn't (for implicit & long cls) enqueue
@@ -275,7 +288,7 @@ end:
     runStats.zeroDepthAssigns += solver->trail_size() - origTrailSize;
     runStats.cpu_time = time_used;
     globalStats += runStats;
-    if (solver->conf.verbosity  >= 1) {
+    if (solver->conf.verbosity) {
         if (solver->conf.verbosity  >= 3)
             runStats.print(solver->nVars());
         else
@@ -290,7 +303,9 @@ end:
     }
 
     if (solver->okay()) {
+        #ifdef DEBUG_ATTACH_MORE
         solver->test_all_clause_attached();
+        #endif
         solver->check_wrong_attach();
         #ifdef DEBUG_IMPLICIT_STATS
         solver->check_stats();
@@ -298,7 +313,30 @@ end:
         checkUnsetSanity();
     }
 
-    return solver->ok;
+    return solver->okay();
+}
+
+bool VarReplacer::replace_xor_clauses()
+{
+    for(Xor& x: solver->xorclauses) {
+        for(uint32_t i = 0, end = x.size(); i < end; i++) {
+            assert(x[i] < solver->nVars());
+            Lit l = Lit(x[i], false);
+            if (get_lit_replaced_with_fast(l) != l) {
+                l = get_lit_replaced_with_fast(l);
+                x.rhs ^= l.sign();
+                x[i] = l.var();
+                runStats.replacedLits++;
+            }
+        }
+
+        solver->clean_xor_vars_no_prop(x.get_vars(), x.rhs);
+        if (x.size() == 0 && x.rhs == true) {
+            solver->ok = false;
+        }
+    }
+
+    return solver->okay();
 }
 
 void VarReplacer::newBinClause(
@@ -314,129 +352,15 @@ void VarReplacer::newBinClause(
         && origLit2 < origLit3
     ){
         delayed_attach_bin.push_back(BinaryClause(lit1, lit2, red));
-        (*solver->drat) << lit1 << lit2 << fin;
+        (*solver->drat) << add << lit1 << lit2
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin;
     }
 }
 
-inline void VarReplacer::updateTri(
-    Watched* i
-    , Watched*& j
-    , const Lit origLit1
-    , const Lit origLit2
-    , Lit lit1
-    , Lit lit2
-) {
-    Lit lit3 = i->lit3();
-    Lit origLit3 = lit3;
-    assert(origLit1.var() != origLit3.var());
-    assert(origLit2.var() != origLit3.var());
-    assert(origLit2 < origLit3);
-    assert(solver->value(origLit3) == l_Undef);
-
-    //Update lit3
-    if (get_lit_replaced_with_fast(lit3) != lit3) {
-        lit3 = get_lit_replaced_with_fast(lit3);
-        i->setLit3(lit3);
-        runStats.replacedLits++;
-    }
-
-    bool remove = false;
-
-    //Tautology, remove
-    if (lit1 == ~lit2
-        || lit1 == ~lit3
-        || lit2 == ~lit3
-    ) {
-        remove = true;
-    }
-
-    //All 3 lits are the same
-    if (!remove
-        && lit1 == lit2
-        && lit2 == lit3
-    ) {
-        delayedEnqueue.push_back(lit1);
-        (*solver->drat) << lit1 << fin;
-        remove = true;
-    }
-
-    //1st and 2nd lits are the same
-    if (!remove
-        && lit1 == lit2
-    ) {
-        newBinClause(origLit1, origLit2, origLit3, lit1, lit3, i->red());
-        remove = true;
-    }
-
-    //1st and 3rd lits  OR 2nd and 3rd lits are the same
-    if (!remove
-        && (lit1 == lit3 || (lit2 == lit3))
-    ) {
-        newBinClause(origLit1, origLit2, origLit3, lit1, lit2, i->red());
-        remove = true;
-    }
-
-    if (remove) {
-        impl_tmp_stats.remove(*i);
-
-        //Drat -- Only delete once
-        if (origLit1 < origLit2
-            && origLit2 < origLit3
-        ) {
-            (*solver->drat)
-            << del
-            << origLit1
-            << origLit2
-            << origLit3
-            << fin;
-        }
-
-        return;
-    }
-
-    //Order literals
-    orderLits(lit1, lit2, lit3);
-
-    //Now make into the order this TRI was in
-    if (origLit1 > origLit2
-        && origLit1 < origLit3
-    ) {
-        std::swap(lit1, lit2);
-    }
-    if (origLit1 > origLit2
-        && origLit1 > origLit3
-    ) {
-        std::swap(lit1, lit3);
-        std::swap(lit2, lit3);
-    }
-    i->setLit2(lit2);
-    i->setLit3(lit3);
-
-    //Drat
-    if (//Changed
-        (lit1 != origLit1
-            || lit2 != origLit2
-            || lit3 != origLit3
-        )
-        //Remove&attach only once
-        && (origLit1 < origLit2
-            && origLit2 < origLit3
-        )
-    ) {
-        (*solver->drat)
-        << lit1 << lit2 << lit3 << fin
-        << del << origLit1 << origLit2  << origLit3 << fin;
-    }
-
-    if (lit1 != origLit1) {
-        solver->watches[lit1].push(*i);
-    } else {
-        *j++ = *i;
-    }
-
-    return;
-}
-
 inline void VarReplacer::updateBin(
     Watched* i
     , Watched*& j
@@ -450,7 +374,12 @@ inline void VarReplacer::updateBin(
     //Two lits are the same in BIN
     if (lit1 == lit2) {
         delayedEnqueue.push_back(lit2);
-        (*solver->drat) << lit2 << fin;
+        (*solver->drat) << add << lit2
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin;
         remove = true;
     }
 
@@ -477,7 +406,12 @@ inline void VarReplacer::updateBin(
         && (origLit1 < origLit2)
     ) {
         (*solver->drat)
-        << lit1 << lit2 << fin
+        << add << lit1 << lit2
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin
         << del << origLit1 << origLit2 << fin;
     }
 
@@ -496,18 +430,11 @@ void VarReplacer::updateStatsFromImplStats()
     assert(impl_tmp_stats.removedIrredBin % 2 == 0);
     solver->binTri.irredBins -= impl_tmp_stats.removedIrredBin/2;
 
-    assert(impl_tmp_stats.removedRedTri % 3 == 0);
-    solver->binTri.redTris -= impl_tmp_stats.removedRedTri/3;
-
-    assert(impl_tmp_stats.removedIrredTri % 3 == 0);
-    solver->binTri.irredTris -= impl_tmp_stats.removedIrredTri/3;
-
     #ifdef DEBUG_IMPLICIT_STATS
     solver->check_implicit_stats();
     #endif
 
     runStats.removedBinClauses += impl_tmp_stats.removedRedBin/2 + impl_tmp_stats.removedIrredBin/2;
-    runStats.removedTriClauses += impl_tmp_stats.removedRedTri/3 + impl_tmp_stats.removedIrredTri/3;
 
     impl_tmp_stats.clear();
 }
@@ -552,9 +479,6 @@ bool VarReplacer::replaceImplicit()
                 lit1 = get_lit_replaced_with_fast(lit1);
                 runStats.replacedLits++;
                 solver->watches.smudge(origLit2);
-                if (i->isTri()) {
-                    solver->watches.smudge(i->lit3());
-                }
             }
 
             //Update lit2
@@ -565,12 +489,8 @@ bool VarReplacer::replaceImplicit()
                 runStats.replacedLits++;
             }
 
-            if (i->isTri()) {
-                updateTri(i, j, origLit1, origLit2, lit1, lit2);
-            } else {
-                assert(i->isBin());
-                updateBin(i, j, origLit1, origLit2, lit1, lit2);
-            }
+            assert(i->isBin());
+            updateBin(i, j, origLit1, origLit2, lit1, lit2);
         }
         ws.shrink_(i-j);
     }
@@ -588,7 +508,7 @@ bool VarReplacer::replaceImplicit()
     updateStatsFromImplStats();
     solver->watches.clear_smudged();
 
-    return solver->ok;
+    return solver->okay();
 }
 
 /**
@@ -605,7 +525,7 @@ bool VarReplacer::replace_set(vector<ClOffset>& cs)
 
         Clause& c = *solver->cl_alloc.ptr(*i);
         assert(!c.getRemoved());
-        assert(c.size() > 3);
+        assert(c.size() > 2);
 
         bool changed = false;
         (*solver->drat) << deldelay << c << fin;
@@ -635,7 +555,7 @@ bool VarReplacer::replace_set(vector<ClOffset>& cs)
     cs.resize(cs.size() - (i-j));
     assert(!solver->drat->something_delayed());
 
-    return solver->ok;
+    return solver->okay();
 }
 
 Lit* my_lit_find(Clause& cl, const Lit lit)
@@ -694,7 +614,11 @@ bool VarReplacer::handleUpdatedClause(
         c.setRemoved();
         return true;
     }
-    (*solver->drat) << c << fin << findelay;
+    (*solver->drat) << add << c
+    #ifdef STATS_NEEDED
+    << solver->sumConflicts
+    #endif
+    << fin << findelay;
 
     runStats.bogoprops += 3;
     switch(c.size()) {
@@ -719,15 +643,6 @@ bool VarReplacer::handleUpdatedClause(
         runStats.removedLongLits += origSize;
         return true;
 
-    case 3:
-        c.setRemoved();
-        solver->watches.smudge(origLit1);
-        solver->watches.smudge(origLit2);
-
-        solver->attach_tri_clause(c[0], c[1], c[2], c.red());
-        runStats.removedLongLits += origSize;
-        return true;
-
     default:
         Lit* at = my_lit_find(c, origLit1);
         if (at != NULL) {
@@ -838,10 +753,33 @@ bool VarReplacer::handleAlreadyReplaced(const Lit lit1, const Lit lit2)
     //OOps, already inside, but with inverse polarity, UNSAT
     if (lit1.sign() != lit2.sign()) {
         (*solver->drat)
-        << ~lit1 << lit2 << fin
-        << lit1 << ~lit2 << fin
-        << lit1 << fin
-        << ~lit1 << fin;
+        << add << ~lit1 << lit2
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin
+
+        << add << lit1 << ~lit2
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin
+
+        << add << lit1
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin
+
+        << add << ~lit1
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin;
 
         solver->ok = false;
         return false;
@@ -859,14 +797,25 @@ bool VarReplacer::replace_vars_already_set(
 ) {
     if (val1 != val2) {
         (*solver->drat)
-        << ~lit1 << fin
-        << lit1 << fin;
+        << add << ~lit1
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin
+
+        << add << lit1
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin;
 
         solver->ok = false;
     }
 
     //Already set, return with correct code
-    return solver->ok;
+    return solver->okay();
 }
 
 bool VarReplacer::handleOneSet(
@@ -883,7 +832,12 @@ bool VarReplacer::handleOneSet(
             toEnqueue = lit1 ^ (val2 == l_False);
         }
         solver->enqueue(toEnqueue);
-        (*solver->drat) << toEnqueue << fin;
+        (*solver->drat) << add << toEnqueue
+        #ifdef STATS_NEEDED
+        << solver->clauseID++
+        << solver->sumConflicts
+        #endif
+        << fin;
 
         #ifdef STATS_NEEDED
         solver->propStats.propsUnit++;
@@ -891,7 +845,7 @@ bool VarReplacer::handleOneSet(
 
         solver->ok = (solver->propagate<false>().isNULL());
     }
-    return solver->ok;
+    return solver->okay();
 }
 
 /**
@@ -914,8 +868,8 @@ bool VarReplacer::replace(
 
     #ifdef DRAT_DEBUG
     (*solver->drat)
-    << Lit(var1, true)  << " " << (Lit(var2, false) ^ xor_is_true) << fin
-    << Lit(var1, false) << " " << (Lit(var2, true)  ^ xor_is_true) << fin
+    << add << Lit(var1, true)  << (Lit(var2, false) ^ xor_is_true) << fin
+    << add << Lit(var1, false) << (Lit(var2, true)  ^ xor_is_true) << fin
     ;
     #endif
 
@@ -928,8 +882,18 @@ bool VarReplacer::replace(
         return handleAlreadyReplaced(lit1, lit2);
     }
     (*solver->drat)
-    << ~lit1 << lit2 << fin
-    << lit1 << ~lit2 << fin;
+    << add << ~lit1 << lit2
+    #ifdef STATS_NEEDED
+    << solver->clauseID++
+    << solver->sumConflicts
+    #endif
+    << fin
+    << add << lit1 << ~lit2
+    #ifdef STATS_NEEDED
+    << solver->clauseID++
+    << solver->sumConflicts
+    #endif
+    << fin;
 
     //None should be removed, only maybe queued for replacement
     assert(solver->varData[lit1.var()].removed == Removed::none);
@@ -1047,10 +1011,10 @@ bool VarReplacer::add_xor_as_bins(const BinaryXor& bin_xor)
     return true;
 }
 
-bool VarReplacer::replace_if_enough_is_found(const size_t limit, uint64_t* bogoprops_given)
+bool VarReplacer::replace_if_enough_is_found(const size_t limit, uint64_t* bogoprops_given, bool *replaced)
 {
-    solver->xorclauses.clear();
-    solver->clear_gauss();
+    if (replaced)
+        *replaced = false;
 
     scc_finder->performSCC(bogoprops_given);
     if (scc_finder->get_num_binxors_found() < limit) {
@@ -1058,6 +1022,13 @@ bool VarReplacer::replace_if_enough_is_found(const size_t limit, uint64_t* bogop
         return solver->okay();
     }
 
+    #ifdef USE_GAUSS
+    solver->clearEnGaussMatrixes();
+    #endif
+
+    if (replaced)
+        *replaced = true;
+
     const set<BinaryXor>& xors_found = scc_finder->get_binxors();
     for(BinaryXor bin_xor: xors_found) {
         if (!add_xor_as_bins(bin_xor)) {
@@ -1102,32 +1073,54 @@ size_t VarReplacer::mem_used() const
     return b;
 }
 
-void VarReplacer::print_equivalent_literals(std::ostream *os) const
+uint32_t VarReplacer::print_equivalent_literals(bool outer_numbering, std::ostream *os) const
 {
+    uint32_t num = 0;
     vector<Lit> tmpCl;
     for (uint32_t var = 0; var < table.size(); var++) {
         const Lit lit = table[var];
         if (lit.var() == var)
             continue;
 
-        tmpCl.clear();
-        tmpCl.push_back(~lit);
-        tmpCl.push_back(Lit(var, false));
-        std::sort(tmpCl.begin(), tmpCl.end());
+        //They have been renumbered in a way that cannot be dumped
+        Lit lit1;
+        Lit lit2;
+        if (outer_numbering) {
+            lit1 = lit;
+            lit2 = Lit(var, false);
+        } else {
+            lit1 = solver->map_outer_to_inter(lit);
+            lit2 = solver->map_outer_to_inter(Lit(var, false));
 
-        *os
-        << tmpCl[0] << " "
-        << tmpCl[1]
-        << " 0\n";
+            if (lit1.var() >= solver->nVars() ||
+                lit2.var() >= solver->nVars()
+            ) {
+                continue;
+            }
+        }
 
-        tmpCl[0] ^= true;
-        tmpCl[1] ^= true;
+        if (os) {
+            tmpCl.clear();
+            tmpCl.push_back(~lit1);
+            tmpCl.push_back(lit2);
+            std::sort(tmpCl.begin(), tmpCl.end());
 
-        *os
-        << tmpCl[0] << " "
-        << tmpCl[1]
-        << " 0\n";
+            *os
+            << tmpCl[0] << " "
+            << tmpCl[1]
+            << " 0\n";
+
+            tmpCl[0] ^= true;
+            tmpCl[1] ^= true;
+
+            *os
+            << tmpCl[0] << " "
+            << tmpCl[1]
+            << " 0\n";
+        }
+        num++;
     }
+    return num;
 }
 
 void VarReplacer::print_some_stats(const double global_cpu_time) const
@@ -1178,10 +1171,6 @@ void VarReplacer::Stats::print(const size_t nVars) const
             , removedBinClauses
         );
 
-        print_stats_line("c tri cls removed"
-            , removedTriClauses
-        );
-
         print_stats_line("c long cls removed"
             , removedLongClauses
         );
@@ -1203,7 +1192,6 @@ void VarReplacer::Stats::print_short(const Solver* solver) const
     << " vars " << actuallyReplacedVars
     << " lits " << replacedLits
     << " rem-bin-cls " << removedBinClauses
-    << " rem-tri-cls " << removedTriClauses
     << " rem-long-cls " << removedLongClauses
     << " BP " << bogoprops/(1000*1000) << "M"
     << solver->conf.print_times(cpu_time)
@@ -1218,7 +1206,6 @@ VarReplacer::Stats& VarReplacer::Stats::operator+=(const Stats& other)
     zeroDepthAssigns += other.zeroDepthAssigns;
     actuallyReplacedVars += other.actuallyReplacedVars;
     removedBinClauses += other.removedBinClauses;
-    removedTriClauses += other.removedTriClauses;
     removedLongClauses += other.removedLongClauses;
     removedLongLits += other.removedLongLits;
     bogoprops += other.bogoprops;
@@ -1308,3 +1295,8 @@ void VarReplacer::load_state(SimpleInFile& f)
         reverseTable[v] = point_to;
     }
 }
+
+bool VarReplacer::get_scc_depth_warning_triggered() const
+{
+    return scc_finder->depth_warning_triggered();
+}
