@@ -30,6 +30,10 @@ THE SOFTWARE.
 #include "src/occsimplifier.h"
 using namespace CMSat;
 #include "test_helper.h"
+#include "src/toplevelgaussabst.h"
+#ifdef USE_M4RI
+#include "src/toplevelgauss.h"
+#endif
 
 struct xor_finder : public ::testing::Test {
     xor_finder()
@@ -57,6 +61,7 @@ TEST_F(xor_finder, find_none)
 
     occsimp->setup();
     XorFinder finder(occsimp, s);
+    finder.grab_mem();
     finder.find_xors();
     EXPECT_EQ(finder.xors.size(), 0U);
 }
@@ -145,7 +150,7 @@ TEST_F(xor_finder, find_4_4)
 /*
  * These tests only work if the matching is non-exact
  * i.e. if size is not checked for equality
- *
+ */
 TEST_F(xor_finder, find_4_2)
 {
     s->add_clause_outer(str_to_cl("-1, 2, 3, 4"));
@@ -182,6 +187,8 @@ TEST_F(xor_finder, find_4_3)
     check_xors_eq(finder.xors, "1, 2, 3, 4 = 0;");
 }
 
+/*
+//Finder pruning is too strong and we don't find this one
 TEST_F(xor_finder, find_5_2)
 {
     s->add_clause_outer(str_to_cl("-1, -2, 3, 4, 5"));
@@ -211,7 +218,7 @@ TEST_F(xor_finder, find_5_2)
     XorFinder finder(occsimp, s);
     finder.find_xors();
     check_xors_eq(finder.xors, "1, 2, 3, 4, 5 = 1;");
-}
+}*/
 
 TEST_F(xor_finder, find_4_5)
 {
@@ -239,6 +246,8 @@ TEST_F(xor_finder, find_4_5)
     finder.find_xors();
     check_xors_eq(finder.xors, "1, 2, 3, 4 = 1; 1, 2, 3, 4 = 0");
 }
+/***
+ * Specialty, non-matching XOR test end
 */
 
 TEST_F(xor_finder, find_5_1)
@@ -273,7 +282,9 @@ TEST_F(xor_finder, find_5_1)
     check_xors_eq(finder.xors, "1, 2, 3, 4, 5 = 1;");
 }
 
-TEST_F(xor_finder, find_6_0)
+
+//we don't find 6-long, too expensive
+/*TEST_F(xor_finder, find_6_0)
 {
     s->add_clause_outer(str_to_cl("1, -7, -3, -4, -5, -9"));
     s->add_clause_outer(str_to_cl("-1, 7, -3, -4, -5, -9"));
@@ -353,195 +364,204 @@ TEST_F(xor_finder, find_6_1)
     XorFinder finder(occsimp, s);
     finder.find_xors();
     check_xors_eq(finder.xors, "6, 7, 3, 4, 5, 9 = 1;");
+}*/
+
+struct xor_finder2 : public ::testing::Test {
+    xor_finder2()
+    {
+        must_inter.store(false, std::memory_order_relaxed);
+        SolverConf conf;
+        conf.doCache = false;
+        s = new Solver(&conf, &must_inter);
+        s->new_vars(30);
+        occsimp = s->occsimplifier;
+        finder = new XorFinder(occsimp, s);
+        finder->grab_mem();
+        #ifdef USE_M4RI
+        topLevelGauss = new TopLevelGauss(s);
+        #endif
+    }
+    ~xor_finder2()
+    {
+        delete s;
+        delete finder;
+        #ifdef USE_M4RI
+        delete topLevelGauss;
+        #endif
+    }
+    Solver* s = NULL;
+    OccSimplifier* occsimp = NULL;
+    std::atomic<bool> must_inter;
+    XorFinder* finder;
+    #ifdef USE_M4RI
+    TopLevelGaussAbst *topLevelGauss;
+    #endif
+};
+
+
+TEST_F(xor_finder2, clean_v1)
+{
+    finder->xors = str_to_xors("1, 2, 3 = 0;");
+    finder->xors = finder->remove_xors_without_connecting_vars(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 0u);
 }
 
-
-TEST_F(xor_finder, clean_v1)
+TEST_F(xor_finder2, clean_v2)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0;");
-    finder.clean_up_xors();
-    EXPECT_EQ(finder.xors.size(), 0u);
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0");
+    finder->xors = finder->remove_xors_without_connecting_vars(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 2u);
 }
 
-TEST_F(xor_finder, clean_v2)
+TEST_F(xor_finder2, clean_v3)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0");
-    finder.clean_up_xors();
-    EXPECT_EQ(finder.xors.size(), 2u);
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0; 10, 11, 12, 13 = 1");
+    finder->xors = finder->remove_xors_without_connecting_vars(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 2u);
 }
 
-TEST_F(xor_finder, clean_v3)
+TEST_F(xor_finder2, clean_v4)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0; 10, 11, 12, 13 = 1");
-    finder.clean_up_xors();
-    EXPECT_EQ(finder.xors.size(), 2u);
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0; 10, 11, 12, 13 = 1; 10, 15, 16, 17 = 0");
+    finder->xors = finder->remove_xors_without_connecting_vars(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 4u);
 }
 
-TEST_F(xor_finder, clean_v4)
+TEST_F(xor_finder2, xor_1)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0; 10, 11, 12, 13 = 1; 10, 15, 16, 17 = 0");
-    finder.clean_up_xors();
-    EXPECT_EQ(finder.xors.size(), 4u);
+    finder->xors = str_to_xors("1, 2, 3 = 1; 1, 4, 5, 6 = 0;");
+    finder->xor_together_xors(finder->xors);
+    check_xors_eq(finder->xors, "2, 3, 4, 5, 6 = 1;");
 }
 
-TEST_F(xor_finder, xor_1)
+TEST_F(xor_finder2, xor_2)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 1; 1, 4, 5, 6 = 0;");
-    finder.xor_together_xors();
-    check_xors_eq(finder.xors, "2, 3, 4, 5, 6 = 1;");
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0;");
+    finder->xor_together_xors(finder->xors);
+    check_xors_eq(finder->xors, "2, 3, 4, 5, 6 = 0;");
 }
 
-TEST_F(xor_finder, xor_2)
+TEST_F(xor_finder2, xor_3)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0;");
-    finder.xor_together_xors();
-    check_xors_eq(finder.xors, "2, 3, 4, 5, 6 = 0;");
+    finder->xors = str_to_xors("1, 2, 3 = 0; 10, 4, 5, 6 = 0;");
+    finder->xor_together_xors(finder->xors);
+    check_xors_eq(finder->xors, "1, 2, 3 = 0; 10, 4, 5, 6 = 0;");
 }
 
-TEST_F(xor_finder, xor_3)
+TEST_F(xor_finder2, xor_4)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 10, 4, 5, 6 = 0;");
-    finder.xor_together_xors();
-    check_xors_eq(finder.xors, "1, 2, 3 = 0; 10, 4, 5, 6 = 0;");
-}
-
-TEST_F(xor_finder, xor_4)
-{
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0;"
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0;"
         "1, 9, 10, 11 = 0;");
-    finder.xor_together_xors();
-    EXPECT_EQ(finder.xors.size(), 3u);
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 3u);
 }
 
-TEST_F(xor_finder, xor_5)
+TEST_F(xor_finder2, xor_5)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 3 = 0; 1, 4, 5, 6 = 0;"
-        "1, 4, 10, 11 = 0;");
-    finder.xor_together_xors();
-    EXPECT_EQ(finder.xors.size(), 2u);
-    check_xors_contains(finder.xors, "5, 6, 10, 11 = 0");
+    finder->xors = str_to_xors("2, 3, 1 = 0; 1, 5, 6, 4 = 0;" "4, 10, 11 = 0;");
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 1u);
+    check_xors_contains(finder->xors, "2, 3, 5, 6, 10, 11 = 0");
 }
 
-TEST_F(xor_finder, xor_6)
+TEST_F(xor_finder2, xor_6)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2 = 0; 1, 4= 0;"
+    finder->xors = str_to_xors("1, 2 = 0; 1, 4= 0;"
         "6, 7 = 0; 6, 10 = 1");
-    finder.xor_together_xors();
-    EXPECT_EQ(finder.xors.size(), 2u);
-    check_xors_eq(finder.xors, "2, 4 = 0; 7, 10 = 1");
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 2u);
+    check_xors_eq(finder->xors, "2, 4 = 0; 7, 10 = 1");
 }
 
-TEST_F(xor_finder, xor_7)
+TEST_F(xor_finder2, dont_xor_together_when_clash_more_than_one)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2 = 0; 1, 2= 0;");
-    finder.xor_together_xors();
-    EXPECT_EQ(finder.xors.size(), 0u);
+    finder->xors = str_to_xors("1, 2, 3, 4 = 0; 1, 2, 5, 6= 0;");
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 2u);
+    check_xors_eq(finder->xors, "1, 2, 3, 4 = 0; 1, 2, 5, 6= 0;");
 }
 
-TEST_F(xor_finder, xor_8)
+TEST_F(xor_finder2, dont_remove_xors)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2 = 0; 1, 2 = 1;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    finder->xors = str_to_xors("1, 2 = 0; 1, 2= 0;");
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 1U);
+}
+
+TEST_F(xor_finder2, dont_remove_xors2)
+{
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 2, 3= 0;");
+    finder->xor_together_xors(finder->xors);
+    EXPECT_EQ(finder->xors.size(), 1U);
+}
+
+TEST_F(xor_finder2, xor_pure_unit_unsat)
+{
+    finder->xors = str_to_xors("1 = 0; 1 = 1;");
+    finder->xor_together_xors(finder->xors);
+    bool ret = finder->add_new_truths_from_xors(finder->xors);
     EXPECT_FALSE(ret);
 }
 
-TEST_F(xor_finder, xor_unit)
+TEST_F(xor_finder2, xor_8)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2 = 0; 1, 2, 3 = 1;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    finder->xors = str_to_xors("1, 2 = 0; 1 = 1; 2 = 0");
+    bool ret = finder->xor_together_xors(finder->xors);
     EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
-}
+    EXPECT_EQ(finder->xors.size(), 1U);
 
-TEST_F(xor_finder, xor_unit2)
-{
-    XorFinder finder(occsimp, s);
-    s->add_clause_outer(str_to_cl("-3"));
-    finder.xors = str_to_xors("1, 2 = 0; 1, 2, 3 = 1;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    ret = finder->add_new_truths_from_xors(finder->xors);
     EXPECT_FALSE(ret);
 }
 
-TEST_F(xor_finder, xor_binx)
+TEST_F(xor_finder2, xor_unit)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 5 = 0; 1, 2, 3 = 0;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    finder->xors = str_to_xors("1 = 0; 1, 2, 3 = 1; 3 = 1");
+    finder->xor_together_xors(finder->xors);
+    bool ret = finder->add_new_truths_from_xors(finder->xors);
     EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
+    EXPECT_EQ(finder->xors.size(), 0u);
+}
+
+#ifdef USE_M4RI
+TEST_F(xor_finder2, xor_unit2_2)
+{
+    s->add_clause_outer(str_to_cl("-4"));
+    finder->xors = str_to_xors("1, 2, 3 = 0; 1, 2, 3, 4 = 1;");
+    vector<Lit> out_changed_occur;
+    bool ret = topLevelGauss->toplevelgauss(finder->xors, &out_changed_occur);
+    EXPECT_FALSE(ret);
+}
+#endif
+
+TEST_F(xor_finder2, xor_binx)
+{
+    finder->xors = str_to_xors("1, 2, 5 = 0; 1, 3 = 0; 2 = 0");
+    bool ret = finder->xor_together_xors(finder->xors);
+    if (ret) {
+        ret &= finder->add_new_truths_from_xors(finder->xors);
+    }
+    EXPECT_TRUE(ret);
+    EXPECT_EQ(finder->xors.size(), 0u);
     check_red_cls_eq(s, "5, -3; -5, 3");
 }
 
-TEST_F(xor_finder, xor_binx_inv)
+TEST_F(xor_finder2, xor_binx_inv_not_found)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 5 = 1; 1, 2, 3 = 0;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    finder->xors = str_to_xors("3, 1 = 1; 1, 3 = 0;");
+    bool ret = finder->xor_together_xors(finder->xors);
+    if (ret) {
+        ret &= finder->add_new_truths_from_xors(finder->xors);
+    }
     EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
-    check_red_cls_eq(s, "-5, -3; 5, 3");
 }
 
-TEST_F(xor_finder, xor_binx_inv2)
+TEST_F(xor_finder2, xor_recur_bug)
 {
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 5 = 1; 1, 2, 3 = 1;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
+    finder->xors = str_to_xors("3, 7, 9 = 0; 1, 3, 4, 5 = 1;");
+    bool ret = finder->xor_together_xors(finder->xors);
     EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
-    check_red_cls_eq(s, "5, -3; -5, 3");
-}
-
-TEST_F(xor_finder, xor_binx2_recur)
-{
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("1, 2, 5 = 0; 2, 3, 4, 5 = 0; 1, 4, 5 = 0;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
-    EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
-    check_red_cls_eq(s, "5, -3; -5, 3");
-}
-
-TEST_F(xor_finder, xor_binx3_recur)
-{
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("8, 9, 2 = 1; 8, 9, 1, 5 = 1; 2, 3, 4, 5 = 0; 1, 4, 5 = 0;");
-    finder.xor_together_xors();
-    bool ret = finder.add_new_truths_from_xors();
-    EXPECT_TRUE(ret);
-    finder.xor_together_xors();
-    ret = finder.add_new_truths_from_xors();
-    EXPECT_TRUE(ret);
-    EXPECT_EQ(finder.xors.size(), 0u);
-    check_red_cls_eq(s, "5, -3; -5, 3");
-}
-
-TEST_F(xor_finder, xor_recur_bug)
-{
-    XorFinder finder(occsimp, s);
-    finder.xors = str_to_xors("3, 7, 9 = 0; 1, 3, 4, 5 = 1;");
-    finder.xor_together_xors();
-    check_xors_eq(finder.xors, "7, 9 , 1, 4, 5 = 1;");
+    check_xors_eq(finder->xors, "7, 9 , 1, 4, 5 = 1;");
 }
 
 
