@@ -4,15 +4,10 @@ var clDistrib = new Array();
 var simplificationPoints = new Array();
 var maxConflRestart = new Array();
 
-//while (true) {
-//setInterval(function(){myajax.makeGetRequest(500005960);}, 2000);
-//myajax.makeGetRequest(86533651);
-//}
-
 function selected_runID(runID) {
     console.log("getting runid " + runID);
     clear_everything();
-    link = "getdata.php?id=" + runID;
+    link = "get_data.php?id=" + runID;
     //runID = "/private/sat/dat/" + runID.replace(/^.*[\\\/]/, '') + ".dat"
     //link = runID;
     jQuery.getJSON(
